// tuneapp.


#ifndef __CLASS_REGISTRY_H__
#define __CLASS_REGISTRY_H__


#include "Item.h"
#include "Category.h"
#include "ConfigException.h"
#include "utils.h"

#include <string>
#include <map>
#include <stdexcept>
using namespace std;


template<typename T>
class ClassRegistry {
public:
	typedef T* (*factory_func_t)();

private:
	map<wstring, factory_func_t> funcMap;

public:
	ClassRegistry() {}

	void registerClass(const wstring& className, factory_func_t func) {
		funcMap[className] = func;
	}

	T* create(const wstring& className) throw(ConfigException) {
		factory_func_t func = utils::find_ptr(funcMap, className);
		if (func == NULL) {
			throw ConfigException(className);
		}
		return func();
	}
};


typedef ClassRegistry<Item> ItemClassRegistry;
typedef ClassRegistry<Category> CategoryClassRegistry;

extern ItemClassRegistry itemClassRegistry;
extern CategoryClassRegistry categoryClassRegistry;

void registerClasses();


#endif // __CLASS_REGISTRY_H__
