// tuneapp.


#ifndef __TUNEAPP_H__
#define __TUNEAPP_H__


#include "lang.h"
#include "Item.h"
#include "Category.h"
#include "ItemGroupSelector.h"
#include "Storage.h"

#include "awt/Container.h"
#include "awt/Label.h"
#include "awt/Choice.h"
#include "awt/Tree.h"
#include "awt/TextArea.h"
#include "awt/Frame.h"
#include "awt/BorderLayout.h"
#include "awt/FlowLayout.h"
#include "awt/GridLayout.h"
#include "awt/GridFlowLayout.h"
#include "awt/FileDialog.h"
#include "awt/event/ActionListener.h"
#include "awt/event/ItemListener.h"
#include "awt/event/WindowListener.h"
#include "awt/event/MouseListener.h"
#include "awt/event/FocusListener.h"
#include "awt/event/KeyListener.h"

#include "ui/RecursiveListener.h"

#include <string>
#include <vector>
#include <map>
using namespace std;


class TuneApp
		: public awt::Frame,
		public awt::WindowListener, public awt::KeyListener {
public:
	static const wstring APP_NAME;
	static const wstring VERSION;
	static const wstring VERSION_SUFIX;

private:
	wstring itemsPath;

	int affect;
	CategoryList categories;

	awt::Tree* categoryTree;
	awt::Choice* affectChoice;
	awt::Container* customizerContainer;
	awt::TextArea* helpTextArea;
	awt::Label* statusBar;

	map<Customizer*, awt::Component*> viewMap;

	awt::BorderLayout* rootLayout;
	awt::GridFlowLayout* groupsLayout;
	awt::BorderLayout* helpLayout;
	awt::FlowLayout* buttonsLayout0;
	awt::GridLayout* buttonsLayout1;
	awt::BorderLayout* statusLayout;

	int eventLockCount;

public:
	TuneApp(const vector<wstring>& args);

	void addNotify();

	void keyPressed(awt::KeyEvent& event);
	void keyReleased(awt::KeyEvent& event);
	void keyTyped(awt::KeyEvent& event);
	void windowClosing(awt::WindowEvent& event);

protected:
	bool eventLocked();
	void lockEvent();
	void unlockEvent();

private:
	void parseArgs(const vector<wstring>& args) throw(ConfigException);
	void setArgument(const wstring& key, const wstring& value) throw(ConfigException);
	void readConfig();
	void createComponents();
	awt::Choice* createAffectSelector();
	awt::Component* createCategoriesTitle();
	awt::Tree* createCategoryTree();
	awt::Container* createCustomizerContainer();
	void addCategoriesView(const CategoryList& categories, awt::Container* container);
	awt::Container* createGroupSelectorContainer(ItemGroupSelector* selector);
	awt::Container* createCategoryContainer(Category* category);
	awt::Component* createCaption(const wstring& text);
	awt::Component* createGroupView(ItemGroup* group);
	awt::TextArea* createHelpTextArea();
	awt::Container* createButtonPane();
	awt::Label* createStatusBar();
	void loadIcon();

	void categoryTreeChanged();
	void affectSelectorChanged();

	void setAffect(int affect);
	void updateViewAffection(const CategoryList& categories);
	void updateCategoryTree();
	void updateCategoryTree(awt::Tree* tree);
	void updateCategoryTreeAffection(awt::Tree* tree, awt::Tree::Node* parent, const CategoryList& categoryList);

	void showCategory(Category* category) throw(IllegalArgumentException);
	void showCategoryInc(int step);
	awt::Tree::Node* getTreeNodeFor(Category* category) throw(IllegalArgumentException);

	void showHelp(const Customizer* target);
	void setHelpText(const wstring& helpText);
	void setStatus(const wstring& status);

	void about();
	void playMp3();

	void ok();
	void apply();
	void cancel();
	void save();

	void showNoPrivileges(const Item* item, const Exception& e);
	void requestLogoff();
	void requestReboot();
	void exit(const wstring& message, int mode);
	void adjustShutdownPrivilige() throw(SecurityException);
	void exitWindows(int mode) throw(SecurityException);

	wstring showSaveDialog();
	wstring showFileDialog(const wstring& title, awt::FileDialog::Type mode, const wstring& name, const wstring& ext);

	class ItemGroupSelectorListener : public awt::ItemListener {
	private:
		TuneApp* const tuneapp;
		vector<wstring> keys;
		awt::Choice* choice;
		awt::Container* selectionContainer;
	public:
		ItemGroupSelectorListener(
				TuneApp* tuneapp,
				const vector<wstring>& keys,
				awt::Choice* choice, awt::Container* selectionContainer);
		void itemStateChanged(awt::ItemEvent& event);
	};
	friend class ItemGroupSelectorListener;

	class HelpKicker : public RecursiveListener,
			public awt::MouseListener, public awt::FocusListener, public awt::ActionListener {
	private:
		TuneApp* const tuneapp;
		Customizer* customizer;
		bool entered;
		bool popupScheduled;
	public:
		HelpKicker(TuneApp* tuneapp, Customizer* customizer);
		UINT getMouseHoverTime();
	protected:
		void installDispatcherTo(awt::Component* component);
		void uninstallDispatcherFrom(awt::Component* component);
	public:
		void focusGained(awt::FocusEvent& event);
		void mousePressed(awt::MouseEvent& event);
		void mouseEntered(awt::MouseEvent& event);
		void mouseExited(awt::MouseEvent& event);
		void actionPerformed(awt::ActionEvent& event);
	};
	friend class HelpKicker;

	class Applyer {
	private:
		TuneApp* const tuneapp;
	public:
		Item::PostChangedOperation pco;
		Applyer(TuneApp* tuneapp) : tuneapp(tuneapp), pco(Item::PCO_NO) {}
		void operator()(Item* item);
	};
	friend class Applyer;

	class Saver {
	private:
		TuneApp* const tuneapp;
		Storage* const storage;
	public:
		Saver(TuneApp* tuneapp, Storage* storage) : tuneapp(tuneapp), storage(storage) {}
		void operator()(Item* item);
	};
	friend class Saver;
};


#endif // __TUNEAPP_H__
