// tuneapp.


#ifndef __VERSION_DEPENDED_DOCUMENT_BUILDER_H__
#define __VERSION_DEPENDED_DOCUMENT_BUILDER_H__


#include "ConfigException.h"

#include "xml/ParserCallback.h"
#include "xml/Document.h"
#include "xml/Node.h"

#include <string>
#include <stack>
using namespace std;


class VersionDependedDocumentBuilder : public xml::ParserCallback {
private:
	enum ProductType {
		PERSONAL, // XP Home
		WORKSTATION, // NT4 Workstation, 2000 Professional, XP Professional
		SERVER // NT4 Server series, 2000 Server series, 2003 Server series
	} productType;
	int productVersion; // NT4 = 0x0400, 2000(NT5.0) = 0x0500, XP(NT5.1) = 0x0501
	int productSPVersion; // SP4 = 0x0400, SP6a = 0x0601
	enum Expression {
		EQUAL, // ==
		LESS, // <
		GREAT_EQUAL // >=
	};

	xml::Document* const document;
	xml::Node* currentNode;
	bool skipping;
	stack<wstring> skippingStack;

public:
	VersionDependedDocumentBuilder(xml::Document* document);
	virtual void startElement(const wstring& name, const Attributes& attrs);
	virtual void endElement(const wstring& name);
	virtual void text(const wstring& text);
	virtual void endDocument();

private:
	void initVersionInfo();
	bool checkVersionDepended(xml::Node* root) throw(ConfigException);
	pair<Expression, wstring> parseSpec(const wstring& spec) throw(ConfigException);
	ProductType getProductType(const wstring& typeString) throw(ConfigException);
	int getProductVersion(const wstring& versionString) throw(ConfigException);
	int getProductSPVersion(const wstring& versionString) throw(ConfigException);
	bool operate(int value1, Expression expression, int value2);
};


#endif // __VERSION_DEPENDED_DOCUMENT_BUILDER_H__
