// AWT less.


#include "BorderLayout.h"

#include <assert.h>


namespace awt {


const LayoutManager::Constraints BorderLayout::NORTH;
const LayoutManager::Constraints BorderLayout::SOUTH;
const LayoutManager::Constraints BorderLayout::EAST;
const LayoutManager::Constraints BorderLayout::WEST;
const LayoutManager::Constraints BorderLayout::CENTER;


BorderLayout::BorderLayout()
		: hgap(0), vgap(0),
		north(NULL), south(NULL), east(NULL), west(NULL), center(NULL) {
}

void BorderLayout::setHgap(int hgap) {
	this->hgap = hgap;
}

void BorderLayout::setVgap(int vgap) {
	this->vgap = vgap;
}

void BorderLayout::addLayoutComponent(Component* child, const Constraints& constraints)
		throw (IllegalArgumentException) {
	if (&constraints == &NORTH) {
		north = child;
	} else if (&constraints == &SOUTH) {
		south = child;
	} else if (&constraints == &EAST) {
		east = child;
	} else if (&constraints == &WEST) {
		west = child;
	} else if (&constraints == &CENTER) {
		center = child;
	} else {
		throw IllegalArgumentException(L"constraints");
	}
}

void BorderLayout::removeLayoutComponent(Component* child) {
	throw Exception(L"Not Implemented");
}

static inline Dimension getPreferredSize(Component* comp) {
	return ((comp == NULL || !comp->isVisible()) ? Dimension() : comp->getPreferredSize());
}

static inline int max3(int a, int b, int c) {
	return max(max(a, b), c);
}

Dimension BorderLayout::getPreferredLayoutSize(Container* parent) {
	Dimension northSize = getPreferredSize(north);
	Dimension southSize = getPreferredSize(south);
	Dimension eastSize = getPreferredSize(east);
	Dimension westSize = getPreferredSize(west);
	Dimension centerSize = getPreferredSize(center);
	Insets insets = parent->getInsets();

	int width
			= max3(	northSize.width,
					southSize.width,
					westSize.width + centerSize.width + eastSize.width)
			+ insets.left + insets.right;
	int height
			=	northSize.height + southSize.height
			+ max3(westSize.height, centerSize.height, eastSize.height)
			+ insets.top + insets.bottom;
	int hc = (westSize.width > 0) + (centerSize.width > 0) + (eastSize.width > 0);
	int vc = (northSize.height > 0) + (westSize.height + centerSize.height + eastSize.height > 0) + (southSize.height > 0);
	if (hc >= 2) {
		width += hgap * (hc - 1);
	}
	if (vc >= 2) {
		height += vgap * (vc - 1);
	}
	Dimension distance = parent->getClientDistance();
	return Dimension(width + distance.width, height + distance.height);
}

static inline bool isVisible(Component* comp) {
	return ((comp != NULL) && comp->isVisible());
}

void BorderLayout::layoutContainer(Container* parent) {
	Insets insets = parent->getInsets();
	Dimension parentSize = parent->getClientSize();
	int rtop = insets.top;
	int rleft = insets.left;
	int rright = parentSize.width - insets.right;
	int rbottom = parentSize.height - insets.bottom;
	if (isVisible(north)) {
		Dimension size(north->getPreferredSize());
		north->setBounds(rleft, rtop, rright - rleft, size.height);
		rtop += size.height + vgap;
	}
	if (isVisible(south)) {
		Dimension size(south->getPreferredSize());
		south->setBounds(rleft, rbottom - size.height, rright - rleft, size.height);
		rbottom -= size.height + vgap;
	}
	if (isVisible(west)) {
		Dimension size(west->getPreferredSize());
		west->setBounds(rleft, rtop, size.width, rbottom - rtop);
		rleft += size.width + hgap;
	}
	if (isVisible(east)) {
		Dimension size(east->getPreferredSize());
		east->setBounds(rright - size.width, rtop, size.width, rbottom - rtop);
		rright -= size.width + hgap;
	}
	if (isVisible(center)) {
		Dimension size(center->getPreferredSize());
		center->setBounds(rleft, rtop, rright - rleft, rbottom - rtop);
	}
}


} // namespace
