// AWT less.


#include "BorderedPanel.h"

#include <assert.h>


namespace awt {


BorderedPanel::BorderedPanel(const wstring& label)
			: label(label) {
	insetsCache.top = -1;
}

LPCTSTR BorderedPanel::getWindowClass() {
	return L"BUTTON";
}

LPCTSTR BorderedPanel::getWindowText() {
	return label.c_str();
}

DWORD BorderedPanel::getWindowStyle() {
	return BS_GROUPBOX | BS_LEFT | BS_TOP | BS_NOTIFY |
			Container::getWindowStyle(); // ignore style specified by panel
}

LRESULT BorderedPanel::windowProc(UINT uMsg, WPARAM wParam, LPARAM lParam) {
	LRESULT result = 0;
	bool consume = false;
	switch (uMsg) {
	case WM_NCHITTEST:
		result = wmNcHitTest(GET_X_LPARAM(lParam), GET_Y_LPARAM(lParam), consume);
		break;
	case WM_ERASEBKGND:
		result = wmEraseBkgnd(reinterpret_cast<HDC>(wParam), consume);
		break;
	}
	if (consume) {
		return result;
	}
	return Panel::windowProc(uMsg, wParam, lParam);
}

LRESULT BorderedPanel::wmNcHitTest(int x, int y, bool& consume) {
	POINT point = {x, y};
	::ScreenToClient(hwnd, &point);
	if ((point.x >= 0) && (point.x < width) && (point.y >= 0) && (point.y < height)) {
		consume = true;
		return HTCLIENT;
	} else {
		// return HTNOWHERE; ?
		return 0;
	}
}

LRESULT BorderedPanel::wmEraseBkgnd(HDC hdc, bool& consume) {
	// workaround: BS_GROUPBOX don't clear background itself.
	// Not Implemented: exclude children
	HBRUSH bgbrush = reinterpret_cast<HBRUSH>(
			::SendMessage(
					parent->getHwnd(), WM_CTLCOLORBTN,
					reinterpret_cast<WPARAM>(hdc), reinterpret_cast<LPARAM>(hwnd)) );
	if (!bgbrush) return 1;
	RECT rect = { 0, 0, width, height };
	::FillRect(hdc, &rect, bgbrush);
	consume = true;
	return 0;
}

void BorderedPanel::setLabel(const wstring& label) {
	if (this->label != label) {
		this->label = label;
		preferedSizeCache.width = -1;
		if (hwnd) {
			::SetWindowText(hwnd, label.c_str());
		}
	}
}

Insets BorderedPanel::getInsets() const {
	if (insetsCache.top == -1) {
		TEXTMETRIC metrics;
		getTextMetrics(NULL_STRING, NULL, &metrics);
		int borderW = ::GetSystemMetrics(SM_CXEDGE);
		int borderH = ::GetSystemMetrics(SM_CYEDGE);
		insetsCache.top = metrics.tmHeight + metrics.tmExternalLeading + metrics.tmHeight / 2;
		insetsCache.left = borderW + metrics.tmAveCharWidth;
		insetsCache.bottom = borderH + metrics.tmHeight / 2;
		insetsCache.right = borderW + metrics.tmAveCharWidth;
	}
	return insetsCache;
}


} // namespace
