// AWT less.


#include "Button.h"

#include <assert.h>


namespace awt {


Button::Button(const wstring& label) : label(label) {
}

LPCTSTR Button::getWindowClass() {
	return L"BUTTON";
}

LPCTSTR Button::getWindowText() {
	return label.c_str();
}

DWORD Button::getWindowStyle() {
	return WS_TABSTOP |
			BS_TEXT | BS_CENTER | BS_VCENTER |
			BS_NOTIFY |
			Component::getWindowStyle();
}

LRESULT Button::wmCommand(WORD notifyMessage, WORD identifier, bool& consume) {
	switch (notifyMessage) {
	case BN_CLICKED:
		fireActionPerformed();
		consume = true;
		return 0;
	}
	return Component::wmCommand(notifyMessage, identifier, consume);
}

const wstring& Button::getLabel() const {
	return label;
}

void Button::setLabel(const wstring& label) {
	if (this->label != label) {
		this->label = label;
		preferedSizeCache.width = -1;
		if (hwnd) {
			::SetWindowText(hwnd, label.c_str());
		}
	}
}

Dimension Button::getPreferredSize() {
	if (preferedSizeCache.width == -1) {
		Insets insets = getInsets();
		SIZE size;
		TEXTMETRIC metrics;
		getTextMetrics(label, &size, &metrics);
		int borderW = ::GetSystemMetrics(SM_CXEDGE) + ::GetSystemMetrics(SM_CXBORDER);
		int borderH = ::GetSystemMetrics(SM_CYEDGE) + ::GetSystemMetrics(SM_CYBORDER);
		preferedSizeCache.width = size.cx + borderW * 2 + metrics.tmAveCharWidth * 2 + insets.left + insets.right;
		preferedSizeCache.height = size.cy + borderH * 2 + metrics.tmExternalLeading * 2 + insets.top + insets.bottom;
	}
	return preferedSizeCache;
}


} // namespace
