// AWT less.


#include "CardLayout.h"

#include "../utils.h"

#include <assert.h>


namespace awt {


CardLayout::CardLayout() : current(NULL) {
}

void CardLayout::addLayoutComponent(Component* child, const LayoutManager::Constraints& constraints)
		throw (IllegalArgumentException) {
	const wstring& name = dynamic_cast<const CardLayout::Constraints&>(constraints).getName();
	if (name.empty()) {
		throw IllegalArgumentException(L"empty constraints");
	}
	nameMap[name] = child;
	if (!current) {
		current = child;
	}
}

void CardLayout::removeLayoutComponent(Component* child) {
	throw Exception(L"Not Implemented");
}

Dimension CardLayout::getPreferredLayoutSize(Container* parent) {
	Dimension size(0, 0);
	for (	Container::const_iterator children = parent->begin();
			children != parent->end();
			++children) {
		Dimension s((*children)->getPreferredSize());
		size.width = max(size.width, s.width);
		size.height = max(size.height, s.height);
	}
	Insets insets(parent->getInsets());
	Dimension distance = parent->getClientDistance();
	size.width += insets.left + insets.right + distance.width;
	size.height += insets.top + insets.bottom + distance.height;
	return size;
}

void CardLayout::layoutContainer(Container* parent) {
	if (!current) return;
	Dimension size = parent->getClientSize();
	Insets insets = parent->getInsets();
	current->setBounds(
			insets.left,
			insets.top,
			size.width - insets.left - insets.right,
			size.height - insets.top - insets.bottom);
	for (	Container::const_iterator children = parent->begin();
			children != parent->end();
			++children) {
		Component* child = (*children);
		if (child != current) {
			Dimension s(child->getSize());
			child->setLocation(-s.width, -s.height);
		}
	}
}

void CardLayout::show(Container* parent, const wstring& name) {
	Component* child = utils::find_ptr(nameMap, name);
	if (child == NULL) {
		throw IllegalArgumentException(L"invalid name: " + name);
	}
	if (current != child) {
		current = child;
		parent->relayout();
	}
}


CardLayout::Constraints::Constraints(const wstring& name) : name(name) {
}

const wstring& CardLayout::Constraints::getName() const {
	return name;
}


} // namespace
