// AWT less.


#include "Checkbox.h"

#include <assert.h>


namespace awt {


Checkbox::Checkbox(const wstring& label, bool state, CheckboxGroup* group)
		: Button(label), state(state), group(group) {
	if (group) {
		group->radioButtonAdded(this);
	}
}

DWORD Checkbox::getWindowStyle() {
	return BS_LEFT |
			(group == NULL ? BS_AUTOCHECKBOX : BS_AUTORADIOBUTTON) |
			(Button::getWindowStyle() & ~BS_CENTER);
}

HWND Checkbox::createWindow() {
	HWND hwnd = Button::createWindow();
	sendSetCheckMessage(hwnd);
	return hwnd;
}

void Checkbox::sendSetCheckMessage(HWND hwnd) {
	::SendMessage(hwnd, BM_SETCHECK, (state ? BST_CHECKED : BST_UNCHECKED), 0);
}

LRESULT Checkbox::wmCommand(WORD notifyMessage, WORD identifier, bool& consume) {
	switch (notifyMessage) {
	case BN_CLICKED:
		updateState();
		fireItemStateChanged();
		consume = true;
		return 0;
	}
	return Button::wmCommand(notifyMessage, identifier, consume);
}

void Checkbox::updateState() {
	LRESULT result = ::SendMessage(hwnd, BM_GETCHECK, 0, 0);
	switch (result) {
	case BST_CHECKED:
		state = true;
		break;
	case BST_UNCHECKED:
		state = false;
		break;
	default:
		throw Exception(L"illegal check box state: " + toString(result));
	}
}

bool Checkbox::getState() const {
	return state;
}

void Checkbox::setState(bool state) {
	if (this->state != state) {
		this->state = state;
		if (hwnd) {
			sendSetCheckMessage(hwnd);
		}
	}
}

CheckboxGroup* Checkbox::getGroup() const {
	return group;
}

Dimension Checkbox::getPreferredSize() {
	if (preferedSizeCache.width == -1) {
		Insets insets = getInsets();
		SIZE size;
		TEXTMETRIC metrics;
		getTextMetrics(label, &size, &metrics);
		int checkW = ::GetSystemMetrics(SM_CXMENUCHECK);
		int checkH = ::GetSystemMetrics(SM_CYMENUCHECK);
		int focusW = ::GetSystemMetrics(SM_CXBORDER);
		int focusH = ::GetSystemMetrics(SM_CYBORDER);
		preferedSizeCache.width = max(checkW, metrics.tmHeight) + metrics.tmAveCharWidth + size.cx + focusW * 2 + insets.left + insets.right;
		preferedSizeCache.height = max(checkH, metrics.tmHeight + metrics.tmExternalLeading) + focusH * 2 + insets.top + insets.bottom;
	}
	return preferedSizeCache;
}


} // namespace
