// AWT less.


#ifndef __AWT_COMPONENT_H__
#define __AWT_COMPONENT_H__


#include "../lang.h"
#include "types.h"

#include "../util/EventListenerList.h"
#include "event/MouseEvent.h"
#include "event/KeyEvent.h"

#include <string>
using namespace std;

#include <windows.h>
#include <windowsx.h>


namespace awt {


class Container;
class Window;
class Toolkit;
class MouseListener;
class KeyListener;
class FocusListener;
class ActionListener;


class Component {
protected:
	int x;
	int y;
	int width;
	int height;
	bool visible;
	bool enabled;
	DWORD foreground;
	DWORD background;

	Container* parent;

	HWND hwnd;
	HBRUSH backgroundBrush;
	WNDPROC oldWindowProc;

	mutable Toolkit* toolkitCache;
	mutable Insets insetsCache;
	mutable Dimension preferedSizeCache;

private:
	EventListenerList<MouseListener> mouseListeners;
	EventListenerList<KeyListener> keyListeners;
	EventListenerList<FocusListener> focusListeners;

	bool mouseOwner;

private:
	Component(const Component&); // no copy
	Component& operator=(const Component&); // no copy
protected:
	Component();
	virtual ~Component();

	static void initCommonControlEx(DWORD dwICC);

	virtual LPCTSTR getWindowClass() = 0;
	virtual LPCTSTR getWindowText();
	virtual DWORD getWindowStyle();
	virtual DWORD getWindowStyleEx();
	virtual HWND createWindow();

	virtual LRESULT windowProc(UINT uMsg, WPARAM wParam, LPARAM lParam);
	LRESULT defWindowProc(UINT uMsg, WPARAM wParam, LPARAM lParam);
	virtual LRESULT wmCommand(WORD notifyMessage, WORD identifier, bool& consume);
	virtual LRESULT wmNotify(UINT notifyCode, LPARAM lParam, bool& consume);

public:
	Container* getParent() const;
	Window* getRootWindow();

	Point getLocation() const;
	virtual void setLocation(int x, int y);
	inline void setLocation(const Point& location) {
		setLocation(location.x, location.y);
	}
	Dimension getSize() const;
	virtual void setSize(int width, int height);
	inline void setSize(const Dimension& size) {
		setSize(size.width, size.height);
	}
	virtual void setBounds(int x, int y, int width, int height);

	Dimension getClientSize() const;
	Dimension getClientDistance() const;

	virtual Dimension getPreferredSize() = 0;

	virtual Insets getInsets() const;
	virtual void setInsets(const Insets& insets);
	
	virtual void doLayout();

	bool isVisible() const;
	virtual void setVisible(bool visible);
	bool isShowing() const;

	bool isEnabled() const;
	virtual void setEnabled(bool enabled);

	void setForeground(DWORD foreground);
	void setBackground(DWORD background);

	bool isFocusable();
	virtual void requestFocus();
	Component* focusNext();
	Component* focusPrevious();

	virtual Toolkit* getToolkit() const;
	virtual void addNotify();
	virtual void removeNotify();
	HWND getHwnd() const;

/**
 *	schedule this component to layout.
 */
	void relayout();
	void relayoutRoot();
	void repaint();
	virtual void repaint(int x, int y, int width, int height);

	void addMouseListener(MouseListener* listener);
	void removeMouseListener(MouseListener* listener);
	void addKeyListener(KeyListener* listener);
	void removeKeyListener(KeyListener* listener);
	void addFocusListener(FocusListener* listener);
	void removeFocusListener(FocusListener* listener);

protected:
	void setTimer(UINT msec, ActionListener* listener);
	void killTimer(ActionListener* listener);

	void addWindowStyle(DWORD style);
	void removeWindowStyle(DWORD style);
	bool hasWindowStyle(DWORD style);

	Dimension getTextExtentPoint(const wstring& text) const;
	void getTextMetrics(const wstring& text, SIZE* size, TEXTMETRIC* metrics) const;
	void selectFont(HDC hdc) const;

	void repaintThisOnly();
	void repaintThisOnly(int x, int y, int width, int height);

	void fireMouseEvent(MouseEvent::Type type, MouseEvent::Button button, int x, int y, bool& consume);
	void fireKeyEvent(KeyEvent::Type type, int keyCode, bool& consume);
	void fireFocusGained(bool& consume);

private:
	HBRUSH wmCtlColor(UINT uMsg, HDC hdc, bool& consume);
	LRESULT wmMouseButton(MouseEvent::Type type, MouseEvent::Button button, WPARAM wParam, LPARAM lParam, bool& consume);
	LRESULT wmMouseMove(WPARAM wParam, LPARAM lParam, bool& consume);
	void processMouseOwned(int x, int y);
	void processMouseLost(int x, int y);
	LRESULT wmKey(KeyEvent::Type type, WPARAM wParam, LPARAM lParam, bool& consume);
	void processFocusRotation(KeyEvent& event);

	friend class Container;
	friend class Window;
	friend class Toolkit;

#ifdef _DEBUG

public:
	virtual void debug(int indent = 0);

#endif // _DEBUG
};


} // namespace


#endif // __AWT_COMPONENT_H__
