// AWT less.


#ifndef __AWT_CONTAINER_H__
#define __AWT_CONTAINER_H__


#include "Component.h"
#include "LayoutManager.h"
#include "event/ContainerEvent.h"
#include "../util/EventListenerList.h"

#include <vector>
using namespace std;


namespace awt {


class ContainerListener;


class Container : public virtual Component {
public:
	typedef vector<Component*> ComponentList;
	typedef ComponentList::iterator iterator;
	typedef ComponentList::const_iterator const_iterator;

private:
	ComponentList childList;
	LayoutManager* layout;

	EventListenerList<ContainerListener> containerListners;

protected:
	Container();
	virtual ~Container();

	virtual DWORD getWindowStyle();

	virtual LRESULT windowProc(UINT uMsg, WPARAM wParam, LPARAM lParam);

public:
	virtual void add(Component* child, const LayoutManager::Constraints& constraints = LayoutManager::Constraints());
	virtual void remove(Component* child);
	Component* getComponent(int index) const;
	int getComponentCount() const;

	const ComponentList& getComponentList() const;
	const_iterator begin() const;
	const_iterator end() const;

	LayoutManager* getLayout() const;
	virtual void setLayout(LayoutManager* layout);

	virtual Dimension getPreferredSize();
	virtual void doLayout();

	virtual void requestFocus();

	virtual void addNotify();
	virtual void removeNotify();

	using Component::repaint;
	virtual void repaint(int x, int y, int width, int height);

	void addContainerListener(ContainerListener* listener);
	void removeContainerListener(ContainerListener* listener);

protected:
	void fireContainerEvent(ContainerEvent::Type type, Component* child);

private:
	Component* findFocusableChildren(Component* component);

#ifdef _DEBUG

public:
	virtual void debug(int indent = 0);

#endif // _DEBUG
};


} // namespace


#endif // __AWT_CONTAINER_H__
