// AWT less.


#include "FileDialog.h"

#include <assert.h>


namespace awt {


FileDialog::FileDialog(Window* parent, const wstring& title, Type type)
		: parent(parent), title(title), type(type) {
}

void FileDialog::setFile(const wstring& fileName) {
	this->fileName = fileName;
}

void FileDialog::setExtention(const wstring& extentionName) {
	this->extentionName = extentionName;
}

wstring FileDialog::getFile() const {
	return fileName;
}

void FileDialog::setVisible(bool visible) {
	if (!visible) {
		throw IllegalArgumentException(L"FileDialog::setVisible(false)");
	}

	if (fileName.length() > _MAX_PATH) {
		throw Exception(L"too long: " + fileName);
	}
	wchar_t fileBuffer[_MAX_PATH + 1];
	fileBuffer[fileName.copy(fileBuffer, _MAX_PATH)] = '\0';

	wstring extentionBuffer;
	if (!extentionName.empty()) {
		SHFILEINFO info;
		DWORD_PTR result = ::SHGetFileInfo(
				(L"." + extentionName).c_str(),
				FILE_ATTRIBUTE_NORMAL,
				&info, sizeof(SHFILEINFO),
				SHGFI_USEFILEATTRIBUTES | SHGFI_TYPENAME);
		if (result == FALSE) {
			extentionBuffer += extentionName + L" file";
		} else {
			extentionBuffer += info.szTypeName;
			extentionBuffer += L" (*." + extentionName + L")";
		}
		extentionBuffer += L'\0';
		extentionBuffer += L"*." + extentionName;
		extentionBuffer += L'\0';
	}

	OPENFILENAME info;
	::ZeroMemory(&info, sizeof(OPENFILENAME));
	info.lStructSize = OPENFILENAME_SIZE_VERSION_400;
	info.hwndOwner = parent->getHwnd();
	info.lpstrFilter = (extentionBuffer.empty() ? NULL : extentionBuffer.c_str());
	info.lpstrFile = fileBuffer;
	info.nMaxFile = _MAX_PATH + 1;
	info.lpstrTitle = title.c_str();
	info.Flags = OFN_ENABLESIZING | OFN_HIDEREADONLY | OFN_OVERWRITEPROMPT;
	BOOL result;
	switch (type) {
	case SAVE:
		result = ::GetSaveFileName(&info);
		break;
	case LOAD:
		result = ::GetOpenFileName(&info);
		break;
	}
	if (result) {
		fileName = info.lpstrFile;
	} else {
		DWORD error = ::CommDlgExtendedError();
		if (error == 0) { // user cancel
			fileName = NULL_STRING;
		} else {
			throw Exception(L"::GetOpen/SaveFileName() failed: " + toString(error));
		}
	}
}


} // namespace
