// AWT less.


#include "FlowLayout.h"

#include <assert.h>


namespace awt {


FlowLayout::FlowLayout(Alignment align)
		: align(align), hgap(0), vgap(0) {
}

FlowLayout::FlowLayout(Alignment align, int hgap, int vgap)
		: align(align), hgap(hgap), vgap(vgap) {
}

void FlowLayout::setHgap(int hgap) {
	this->hgap = hgap;
}

void FlowLayout::setVgap(int vgap) {
	this->vgap = vgap;
}

void FlowLayout::addLayoutComponent(Component* child, const Constraints& constraints) {
}

void FlowLayout::removeLayoutComponent(Component* child) {
}

Dimension FlowLayout::getPreferredLayoutSize(Container* parent) {
	Dimension size(0, 0);
	for (	Container::const_iterator children = parent->begin();
			children != parent->end();
			++children) {
		Component* child = *children;
		if (child->isVisible()) {
			if (size.width > 0) {
				size.width += hgap;
			}
			Dimension s(child->getPreferredSize());
			size.width += s.width;
			size.height = max(size.height, s.height);
		}
	}
	Insets insets = parent->getInsets();
	size.width += insets.left + insets.right;
	size.height += insets.top + insets.bottom;
	Dimension distance = parent->getClientDistance();
	size.width += distance.width;
	size.height += distance.height;
	return size;
}

void FlowLayout::layoutContainer(Container* parent) {
	// Not Implemented: wrapping

	Dimension size = parent->getClientSize();
	Insets insets = parent->getInsets();

	int totalWidth = 0;
	vector< pair<Component*, Dimension> > layouteeSizeList;
	// vector<int> lineHeights;
	layouteeSizeList.reserve(parent->getComponentCount());
	for (	Container::const_iterator children = parent->begin();
			children != parent->end();
			++children) {
		Component* child = *children;
		if (child->isVisible()) {
			if (totalWidth > 0) {
				totalWidth += hgap;
			}
			Dimension s = child->getPreferredSize();
			totalWidth += s.width;
			layouteeSizeList.push_back(make_pair(child, s));
		}
	}

	int x;
	switch (align) {
	case LEFT:
		x = insets.left;
		break;
	case CENTER:
		x = insets.left + (size.width - insets.left - insets.right - totalWidth) / 2;
		break;
	case RIGHT:
		x = size.width - insets.right - totalWidth;
		break;
	default:
		assert (false);
	}
	if (x < insets.left) {
		x = insets.left;
	}
	int y = insets.top;
	size.height -= insets.top + insets.bottom;
	for (	vector< pair<Component*, Dimension> >::const_iterator iterator = layouteeSizeList.begin();
			iterator != layouteeSizeList.end();
			++iterator) {
		Component* child = (*iterator).first;
		Dimension s = (*iterator).second;
		child->setBounds(x, y + (size.height - s.height) / 2, s.width, s.height);
		x += s.width + hgap;
	}
}


} // namespace
