// AWT less.


#include "GridLayout.h"


namespace awt {


GridLayout::GridLayout(int rows, int cols)
		: rows(rows), cols(cols), hgap(0), vgap(0) {
	if ((rows == 0) && (cols == 0)) {
		throw IllegalArgumentException(L"both of rows and cols specified 0");
	}
}

void GridLayout::setHgap(int hgap) {
	this->hgap = hgap;
}

void GridLayout::setVgap(int vgap) {
	this->vgap = vgap;
}

void GridLayout::addLayoutComponent(Component* child, const Constraints& constraints) {
}

void GridLayout::removeLayoutComponent(Component* child) {
}

Dimension GridLayout::getPreferredLayoutSize(Container* parent) {
	int rows = getRowsCount(parent);
	int cols = getColsCount(parent);
	int width = 0;
	int height = 0;
	for (	Container::const_iterator children = parent->begin();
			children != parent->end();
			++children) {
		Component* child = (*children);
		if (child->isVisible()) {
			Dimension s = child->getPreferredSize();
			width = max(width, s.width);
			height = max(height, s.height);
		}
	}
	width *= cols;
	height *= rows;
	Insets insets = parent->getInsets();
	Dimension distance = parent->getClientDistance();
	return Dimension(
			width + insets.left + insets.right + hgap * (cols - 1) + distance.width,
			height + insets.top + insets.bottom + vgap * (rows -1) + distance.height);
}

void GridLayout::layoutContainer(Container* parent) {
	Dimension size = parent->getClientSize();
	Insets insets = parent->getInsets();
	int rows = getRowsCount(parent);
	int cols = getColsCount(parent);
	int width = size.width - insets.left - insets.right - hgap * (cols - 1);
	int height = size.height - insets.top - insets.bottom - vgap * (rows - 1);
	for (int i = 0, r = 0; r < rows; r++) {
		for (int c = 0; c < cols; c++, i++) {
			Component* child = parent->getComponent(i);
			if (child->isVisible()) {
				int x = width * c / cols;
				int y = height * r / rows;
				int w = width * (c + 1) / cols - x;
				int h = height * (r + 1) / rows - y;
				x += hgap * c;
				y += vgap * r;
				child->setBounds(insets.left + x, insets.top + y, w, h);
			}
		}
	}
}

int GridLayout::getRowsCount(Container* parent) {
	if (rows == 0) {
		return (parent->getComponentCount() + cols - 1) / cols;
	} else {
		return rows;
	}
}

int GridLayout::getColsCount(Container* parent) {
	if (cols == 0) {
		return (parent->getComponentCount() + rows - 1) / rows;
	} else {
		return cols;
	}
}


} // namespace
