// AWT less.


#ifndef __AWT_NUMBER_FIELD_H__
#define __AWT_NUMBER_FIELD_H__


#include "Panel.h"
#include "TextField.h"
#include "event/AdjustmentEventSource.h"
#include "event/KeyListener.h"
#include "event/TextListener.h"
#include "event/ActionListener.h"


namespace awt {


class NumberField : public TextField, public AdjustmentEventSource {
private:
	class Handler;
protected:
	Handler * handler;

	int value;
	int minimum;
	int maximum;
	int unitIncStep;
	int blockIncStep;

protected:
	virtual DWORD getWindowStyle();
	virtual HWND createWindow();

public:
	NumberField(int value = 0);
	virtual ~NumberField();

	int getMinimum();
	void setMinimum(int minimum);
	int getMaximum();
	void setMaximum(int maximum);
	int getUnitIncrement();
	void setUnitIncrement(int unitIncStep);
	int getBlockIncrement();
	void setBlockIncrement(int blockIncStep);

	int getValue();
	void setValue(int value, AdjustmentEvent::Type adjustmentType = AdjustmentEvent::TRACK);

	virtual Dimension getPreferredSize();

private:
	void unitInc();
	void unitDec();
	void blockInc();
	void blockDec();

	int fixValue(int value);
	void updateText();
	void updateValue();

private:
	class Handler : public KeyListener, public TextListener, public ActionListener {
	private:
		NumberField* const target;
		bool recovering;
	public:
		Handler(NumberField* target);
		void keyPressed(KeyEvent& event);
		void keyReleased(KeyEvent& event);
		void keyTyped(KeyEvent& event);
		void textValueChanged(TextEvent& event);
		void actionPerformed(ActionEvent& event);
	};
	friend class Handler;
};


} // namespace


#endif // __AWT_NUMBER_FIELD_H__
