// AWT less.


#include "TextArea.h"

#include <assert.h>


namespace awt {


TextArea::TextArea(const wstring& text, int rows, int cols)
		: TextComponent(text),
		rows(rows), cols(cols),
		lineSpace(0),
		dll(NULL) {
}

LPCTSTR TextArea::getWindowClass() {
	return RICHEDIT_CLASS;
}

DWORD TextArea::getWindowStyle() {
	return ES_MULTILINE | WS_VSCROLL | ES_AUTOVSCROLL |
			TextComponent::getWindowStyle();
}

HWND TextArea::createWindow() {
	dll = ::LoadLibrary(L"riched20.dll");
	if (dll == NULL) {
		throw Exception(L"LoadLibrary(riched20.dll) failed: " + toString(::GetLastError()));
	}
	HWND hwnd = TextComponent::createWindow();
	// ::SendMessage(hwnd, EM_SETLANGOPTIONS, 0, IMF_UIFONTS);
	::SendMessage(hwnd, EM_SETBKGNDCOLOR, 0, ::GetSysColor(COLOR_3DFACE));
	sendSetLineSpaceMessage(hwnd);
	return hwnd;
}

void TextArea::sendSetLineSpaceMessage(HWND hwnd) {
	PARAFORMAT2 info;
	::ZeroMemory(&info, sizeof(PARAFORMAT2));
	info.cbSize = sizeof(PARAFORMAT2);
	info.dwMask = PFM_LINESPACING;
	info.dyLineSpacing = lineSpace;
	info.bLineSpacingRule = 3;
	LRESULT result = ::SendMessage(
			hwnd, EM_SETPARAFORMAT, 0, reinterpret_cast<LPARAM>(&info));
	if (result == FALSE) {
		throw Exception(L"SendMessage(EM_SETPARAFORMAT) failed");
	}
}

void TextArea::setLineSpace(int lineSpace) {
	this->lineSpace = lineSpace;
	if (hwnd) {
		sendSetLineSpaceMessage(hwnd);
	}
}

Dimension TextArea::getPreferredSize() {
	if (preferedSizeCache.width == -1) {
		assert (hwnd != NULL);
		auto_hdc hdc(hwnd);
		// selectFont(hdc);
		HGDIOBJ font = ::GetStockObject(DEFAULT_GUI_FONT);
		::SelectObject(hdc, font);
		TEXTMETRIC metrics;
		::GetTextMetrics(hdc, &metrics);
		int lineGap = (rows > 1 ? (lineSpace * (rows - 1)) : 0);
		int borderW = ::GetSystemMetrics(SM_CXEDGE) + ::GetSystemMetrics(SM_CXBORDER);
		int borderH = ::GetSystemMetrics(SM_CYEDGE) + ::GetSystemMetrics(SM_CYBORDER);
		int scrollW = ::GetSystemMetrics(SM_CXVSCROLL);
		preferedSizeCache.width = metrics.tmAveCharWidth * cols + scrollW + borderW * 2;
		preferedSizeCache.height = metrics.tmHeight * rows + borderH * 2;
	}
	return preferedSizeCache;
}

void TextArea::removeNotify() {
	TextComponent::removeNotify();
	if (dll) {
		::FreeLibrary(dll);
		dll = NULL;
	}
}


} // namespace
