// AWT less.


#include "TextComponent.h"
#include "event/TextListener.h"

#include "../types.h"

#include <assert.h>


namespace awt {


TextComponent::TextComponent(const wstring& text)
		: text(text), editable(true),
		windowTextUpdated(false) {
}

LPCTSTR TextComponent::getWindowClass() {
	return L"EDIT";
}

LPCTSTR TextComponent::getWindowText() {
	return text.c_str();
}

DWORD TextComponent::getWindowStyle() {
	return WS_TABSTOP |
			ES_LEFT | (editable ? 0 : ES_READONLY) |
			Component::getWindowStyle();
}

DWORD TextComponent::getWindowStyleEx() {
	return WS_EX_CLIENTEDGE |
			Component::getWindowStyleEx();
}

LRESULT TextComponent::wmCommand(WORD notifyMessage, WORD identifier, bool& consume) {
	switch (notifyMessage) {
	case EN_CHANGE:
		windowTextUpdated = true;
		fireTextValueChanged();
		consume = true;
		return 0;
	}
	return Component::wmCommand(notifyMessage, identifier, consume);
}

LRESULT TextComponent::wmNotify(UINT notifyCode, LPARAM lParam, bool& consume) {
	switch (notifyCode) {
	case EN_MSGFILTER:
		MSGFILTER* msg = reinterpret_cast<MSGFILTER*>(lParam);
		windowProc(msg->msg, msg->lParam, msg->wParam);
		break;
	}
	return Component::wmNotify(notifyCode, lParam, consume);
}

const wstring& TextComponent::getText() const {
	if (hwnd && windowTextUpdated) {
		int length = ::GetWindowTextLength(hwnd);
		auto_array<wchar_t> buffer(length + 1);
		length = ::GetWindowText(hwnd, buffer, length + 1);
		text.erase();
		text.append(buffer, length);
		windowTextUpdated = false;
	}
	return text;
}

void TextComponent::setText(const wstring& text) {
	if (hwnd) {
		if ((this->text == text) && !windowTextUpdated) return;
		::SetWindowText(hwnd, text.c_str());
		this->text = text;
		windowTextUpdated = false;
	} else {
		if (this->text == text) return;
		this->text = text;
	}
}

void TextComponent::setCaretPosition(int pos) {
	if (hwnd) {
		::SendMessage(hwnd, EM_SETSEL, pos, pos);
	}
}

bool TextComponent::isEditaboe() const {
	return editable;
}

void TextComponent::setEditable(bool editable) {
	this->editable = editable;
	if (hwnd) {
		LRESULT result = ::SendMessage(hwnd, EM_SETREADONLY, editable, 0);
		if (result == FALSE) {
			throw Exception(L"SendMessage(EM_SETREADONLY) failed");
		}
	}
}

void TextComponent::addTextListener(TextListener* listener) {
	textListeners.add(listener);
}

void TextComponent::removeTextListener(TextListener* listener) {
	textListeners.remove(listener);
}

void TextComponent::fireTextValueChanged() {
	TextEvent event(this);
	FIRE_EVENT(textListeners, TextListener, textValueChanged, TextEvent, event);
}


} // namespace
