// AWT less.


#ifndef __AWT_TOOLKIT_H__
#define __AWT_TOOLKIT_H__


#include "event/KeyListener.h"
#include "../util/EventListenerList.h"

#include "../lang.h"

#include <string>
#include <deque>
using namespace std;

#include <windows.h>


namespace awt {


class Component;


class Toolkit {
private:
	static HINSTANCE hInstance;
	static Toolkit* defaultToolkit;

public:
	static void init(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLine, int nCmdShow);
	static Toolkit* getDefaultToolkit();

private:
	static ATOM frameClass;

private:
	EventListenerList<KeyListener> keyListeners;
	deque<Component*> layoutQueue;

protected:
	Toolkit();
private:
	Toolkit(const Toolkit&);
	Toolkit& operator=(const Toolkit&);
public:
	int messageLoop();

	wstring getStringResource(UINT id);

	HINSTANCE getHinstance();
	ATOM getFrameClass();
	static Component* getComponentByHwnd(HWND hwnd);

	void addKeyListener(KeyListener* listener);
	void removeKeyListener(KeyListener* listener);

private:
	static LRESULT WINAPI WindowProc(HWND hwnd, UINT uMsg, WPARAM wParam, LPARAM lParam);
	LRESULT wmKey(HWND hwnd, KeyEvent::Type type, WPARAM wParam, LPARAM lPrama, bool& consume);
	void fireKeyEvent(Component* component, KeyEvent::Type type, int keyCode, bool& consume);
	ATOM registerClass(const wstring& className, UINT style);

private: // package access
	void scheduleLayout(Component* component);
	void processDelayedLayout();
	static void mapHwnd(HWND hwnd, Component* component);
	static void unmapHwnd(HWND hwnd);
	static void subclass(Component* component);

	friend class Component;
};


} // namespace


#endif // __AWT_TOOLKIT_H__
