// AWT less.


#ifndef __AWT_TREE_H__
#define __AWT_TREE_H__


#include "Component.h"
#include "event/ItemEventSource.h"

#include <vector>
using namespace std;

#include <commctrl.h>


namespace awt {


class Tree : public virtual Component, public virtual ItemEventSource {
public:
	class Node {
		friend class Tree;
	private:
		HTREEITEM handle;
		wstring label;
		void* value;
		bool expanded;
		vector<Node*> childList;
	public:
		Node();
		~Node();
		void* getValue() const;
		bool isExpanded() const;
		int getChildCount() const;
		Node* getChild(int index);
	protected:
		Node* addNode();
		void clearChildren();
	private:
		Node(const Node&);
		Node& operator=(const Node&);
	};

	typedef vector<Node*> NodeList;

private:
	bool showRootHandles;

	Node rootNode;
	Node* selectedNode;

	int eventLockCount;

protected:
	virtual LPCTSTR getWindowClass();
	virtual DWORD getWindowStyle();
	virtual DWORD getWindowStyleEx();
	virtual HWND createWindow();

	virtual LRESULT wmNotify(UINT notifyCode, LPARAM lParam, bool& consume);

public:
	Tree();

	Node* getRootNode();

	Node* addNode(const wstring& label, void* value = NULL);
	Node* addNode(Node* parent, const wstring label, void* value = NULL);
	void removeAll();

	Node* getSelectedNode();
	void select(Node* node);

	Node* getNextNode(Node* node);
	Node* getPreviousNode(Node* node);
	Node* findValue(Node* node, void* value);

	void setShowsRootHandles(bool showRootHandles);
	void setExpanded(Node* node, bool expanded);

	virtual Dimension getPreferredSize();

	Node* getClosestNode(const Point& location) const;

protected:
	bool eventLocked();
	void lockEvent();
	void unlockEvent();

private:
	Node* getNode(HTREEITEM handle) const;
	void getDispInfo(NMTVDISPINFO* info);
	void sendAddMessage(HWND hwnd, const Node& parent, Node& node);
	void sendAddChildrenMessage(HWND hwnd, Node& node);
	void sendSelectMessage(HWND hwnd, const Node& node);
	void sendExpandMessage(HWND hwnd, const Node& node);
	void updateSelectedNode(const TVITEM& newItem);
	void updateNodeExpanded(const TVITEM& newItem, UINT action);
	Node* getNextNode(Node* node, WPARAM wParam);

	int getMaxWidth(HDC hdc, const Node& parent, int handleW, int indent);
};


} // namespace


#endif // __AWT_TREE_H__
