// AWT less.


#include "KeyEvent.h"

#include "../../lang.h"


namespace awt {


const int KeyEvent::SHIFT_MASK = 1;
const int KeyEvent::CTRL_MASK = 2;
const int KeyEvent::ALT_MASK = 4;


KeyEvent::Type KeyEvent::messageToType(UINT uMsg) {
	switch (uMsg) {
	case WM_KEYDOWN:
		return PRESSED;
	case WM_KEYUP:
		return RELEASED;
	case WM_CHAR:
		return TYPED;
	default:
		throw IllegalArgumentException(toString(uMsg));
	}
}

static inline bool checkKeyState(int vkey) {
	return ((::GetKeyState(vkey) & (1 << 15))) ? true : false;
}

static int getModifires() {
	return	(checkKeyState(VK_SHIFT) ? KeyEvent::SHIFT_MASK : 0) |
			(checkKeyState(VK_CONTROL) ? KeyEvent::CTRL_MASK : 0) |
			(checkKeyState(VK_MENU) ? KeyEvent::ALT_MASK : 0);
}

KeyEvent::KeyEvent(Component* source, Type type, int keyCode)
		: EventObject<Component*>(source),
		type(type), keyCode(keyCode), modifires(getModifires()) {
}

KeyEvent::Type KeyEvent::getType() const {
	return type;
}

int KeyEvent::getKeyCode() const {
	return keyCode;
}

int KeyEvent::getModifiers() const {
	return modifires;
}


} // namespace
