// tuneapp.


#ifndef __EXEC_ITEM_H__
#define __EXEC_ITEM_H__


#include "../AbstractItem.h"

#include "../awt/Panel.h"
#include "../awt/Button.h"
#include "../awt/event/ActionListener.h"

#include <map>
using namespace std;


/**
 *	Generic execution item.
 *	Add shortcut files to <code>&lt;tuneapp installed dir&gt;/apps/</code> directory
 *	those you need lounch from tuneapp.
 */
class ExecItem : public AbstractItem {
private:
	map<wstring, wstring> commandMap;

public:
	void setParams(xml::Node* params) throw(ConfigException);
private:
	void setRegistryParams(xml::Node* params) throw(ConfigException);
	void setFileParams(xml::Node* params) throw(ConfigException);
	void setDirectoryParams(xml::Node* params) throw(ConfigException);
	void addCommand(const wstring& name, const wstring& command);

public:
	bool isChanged();
	void setDefault();
	void apply();
	void cancel();
	void store(Storage* storage);
	virtual const wstring& getShortHelpText() const;

	awt::Component* getViewComponent();

private:
	class View : public awt::Panel {
	private:
		ExecItem* const target;
	public:
		View(ExecItem* target);
	private:
		void createComponents();
		awt::Button* createButton(const wstring& label, const wstring& command);
		void exec(const wstring& command);

		class Executioner : public awt::ActionListener {
		private:
			View* const target;
			wstring command;
		public:
			Executioner(View* target, const wstring& command)
					: target(target), command(command) {}
			void actionPerformed(awt::ActionEvent& event);
		};
		friend class Executioner;
	};
	friend class View;
};


#endif // __EXEC_ITEM_H__
