// tuneapp.


#include "lang.h"

#include <algorithm>
#include <sstream>
using namespace std;


const wstring NULL_STRING(1, (wchar_t)(0xffff));


// exception


Exception::Exception(const wstring& message) : message(message) {
}

const wstring& Exception::getMessage() const {
	return message;
}

IllegalArgumentException::IllegalArgumentException(const wstring& message) : Exception(message) {
}

NumberFormatException::NumberFormatException(const wstring& message) : Exception(message) {
}

SecurityException::SecurityException(const wstring& message) : Exception(message) {
}


// java.lang


wstring toString(int value) {
	wostringstream buffer;
	buffer << value;
	return buffer.str();
}

int parseInt(const wstring& string) throw(NumberFormatException) {
	int length = string.length();
	if (length == 0) {
		throw NumberFormatException(L"empty string");
	}
	int i = 0;
	bool negative;
	int value = 0;
	if (string[0] == '-') {
		if (length == 1) {
			throw NumberFormatException(string);
		}
		negative = true;
		i++;
	} else {
		negative = false;
	}
	for (; i < length; i++) {
		wchar_t c = string[i];
		if ((c < '0') || (c > '9')) {
			throw NumberFormatException(string);
		}
		if (value < INT_MIN / 10) {
			throw NumberFormatException(string + L" (overflow)");
		}
		value = value * 10 - (c - '0');
		// architecture depended.
		if (value > 0) {
			throw NumberFormatException(string + L" (overflow)");
		}
	}
	if (negative) {
		return value;
	} else {
		if (value == INT_MIN) {
			throw NumberFormatException(string + L" (overflow)");
		}
		return -value;
	}
}

bool parseBool(const wstring& string) throw (NumberFormatException) {
	return equalsIgnoreCase(string, L"true");
}

wstring toLowerCase(const wstring& src) {
	wstring lower(src);
	transform(src.begin(), src.end(), lower.begin(), tolower);
	return lower;
}

bool startsWith(const wstring& string, const wchar_t* pattern) {
	int len = wcslen(pattern);
	if (string.length() < len) return false;
	return !string.compare(0, len, pattern);
}

bool startsWith(const wstring& string, const wstring& pattern) {
	return !string.compare(0, pattern.length(), pattern);
}

bool startsWithIgnoreCase(const wstring& string, const wchar_t* pattern) {
	int len = wcslen(pattern);
	if (string.length() < len) return false;
	return equalsIgnoreCase(string.begin(), pattern, len);
}

bool endsWith(const wstring& string, const wchar_t* pattern) {
	int len = wcslen(pattern);
	return !string.compare(string.length() - len, len, pattern);
}

bool endsWithIgnoreCase(const wstring& string, const wchar_t* pattern) {
	int len = wcslen(pattern);
	if (string.length() < len) return false;
	return equalsIgnoreCase(string.begin() + string.length() - len, pattern, len);
}

bool equalsIgnoreCase(const wstring& string, const wchar_t* pattern) {
	int len = wcslen(pattern);
	if (string.length() != len) return false;
	return equalsIgnoreCase(string.begin(), pattern, len);
}

wstring trim(const wstring& src) {
	wstring trim(src);
	for (wstring::size_type i = 0; i < trim.length(); i++) {
		if (trim.at(i) > 0x0020) break;
	}
	if (i > 0) {
		trim.erase(0, i);
		if (trim.empty()) {
			return trim;
		}
	}
	for (i = trim.length() - 1; i >= 0; i--) {
		if (trim.at(i) > 0x0020) break;
	}
	if (i < trim.length() - 1) {
		trim.erase(i + 1, trim.length() - i - 1);
	}
	return trim;
}
