// tuneapp.


#include "AbstractRegBitValueItem.h"


AbstractRegBitValueItem::AbstractRegBitValueItem() {
}


void AbstractRegBitValueItem::setParams(xml::Node* params) throw(ConfigException) {
	AbstractRegItem::setParams(params);
	init();
}

void AbstractRegBitValueItem::init() {
	defaultValue = parseInt(getDefaultValueString());
	readRegistry();
}

void AbstractRegBitValueItem::readRegistry() {
	try {
		auto_ptr<RegKey> key(openRegKey(KEY_READ));
		systemValue = readRegistry(key.get());
	} catch (RegKeyException) {
		systemValue = defaultValue;
	}
	currentValue = systemValue;
}

int AbstractRegBitValueItem::readRegistry(RegKey* key) throw(RegKeyException) {
	return key->getIntValue(getRegValueName(), defaultValue);
}

int AbstractRegBitValueItem::getSystemValue() const {
	return systemValue;
}

int AbstractRegBitValueItem::getDefaultValue() const {
	return defaultValue;
}

int AbstractRegBitValueItem::getCurrentValue() const {
	return currentValue;
}

void AbstractRegBitValueItem::setCurrentValue(int currentValue) {
	if (this->currentValue == currentValue) return;
	this->currentValue = currentValue;
	firePropertyChange(PROP_CURRENT_VALUE);
}

void AbstractRegBitValueItem::setDefault() {
	setCurrentValue(defaultValue);
}

void AbstractRegBitValueItem::apply() throw(RegKeyException) {
	if (currentValue == systemValue) return;
	writeRegistry();
	setChanged();
}

void AbstractRegBitValueItem::writeRegistry() throw(RegKeyException) {
	auto_ptr<RegKey> key(openRegKey(KEY_WRITE | KEY_READ));
	writeRegistry(key.get(), currentValue);
	systemValue = currentValue;
}

void AbstractRegBitValueItem::writeRegistry(RegKey* key, int value) throw(RegKeyException) {
	if ((value == defaultValue) && isDeleteForDefault()) {
		key->deleteValue(getRegValueName());
	} else {
		key->setValue(getRegValueName(), value);
	}
}

void AbstractRegBitValueItem::cancel() {
	setCurrentValue(systemValue);
}

void AbstractRegBitValueItem::store(RegistryStorage* storage) {
	RegistryStorage::Key* key = storage->getKey(getRegKeyName());
	if ((currentValue == defaultValue) && isDeleteForDefault()) {
		key->setDeleteValue(getRegValueName());
	} else {
		key->setDWORD(getRegValueName(), currentValue);
	}
}
