// tuneapp.


#include "AbstractRegCheckItem.h"

#include "../Inheritable.h"
#include "../ConfigReader.h"

#include "../ui/CheckItemView.h"
#include "../ui/InheritCheckItemView.h"


AbstractRegCheckItem::AbstractRegCheckItem()
		: defaultValue(false), label(NULL_STRING) {
}

void AbstractRegCheckItem::setParams(xml::Node* params) throw(ConfigException) {
	for (	xml::Node::const_iterator children = params->begin();
			children != params->end();
			++children) {
		xml::Node* child = *children;
		if (child->getType() == xml::Node::ELEMENT) {
			const wstring& name = child->getName();
			if (name == L"label") {
				label = child->getText();
			}
		}
	}
	AbstractRegItem::setParams(params);
}

void AbstractRegCheckItem::readRegistry() {
	try {
		auto_ptr<RegKey> key(openRegKey(KEY_READ));
		systemValue = readRegistry(key.get());
	} catch (RegKeyException) {
		systemValue = defaultValue;
	}
	currentValue = systemValue;
	// make short help
}

awt::Component* AbstractRegCheckItem::getViewComponent() {
	Inheritable* inheritable = dynamic_cast<Inheritable*>(this);
	if (inheritable && inheritable->isInheritor()) {
		return new InheritCheckItemView(this);
	} else {
		return new CheckItemView(this);
	}
}

bool AbstractRegCheckItem::getSystemValue() const {
	return systemValue;
}

bool AbstractRegCheckItem::getCurrentValue() const {
	return currentValue;
}

bool AbstractRegCheckItem::getDefaultValue() const {
	return defaultValue;
}

void AbstractRegCheckItem::setDefaultValue(bool defaultValue) {
	if (this->defaultValue == defaultValue) return;
	this->defaultValue = defaultValue;
	firePropertyChange(PROP_DEFAULT_VALUE);
}

void AbstractRegCheckItem::setCurrentValue(bool value) {
	if (this->currentValue == value) return;
	this->currentValue = value;
	firePropertyChange(PROP_CURRENT_VALUE);
}

const wstring& AbstractRegCheckItem::getLabel() const {
	return label;
}

void AbstractRegCheckItem::setDefault() {
	setCurrentValue(defaultValue);
}

void AbstractRegCheckItem::apply() throw(RegKeyException) {
	if (currentValue == systemValue) return;

	writeRegistry();
	setChanged();
}

void AbstractRegCheckItem::writeRegistry() throw(RegKeyException) {
	auto_ptr<RegKey> key(openRegKey(KEY_WRITE | KEY_READ));
	writeRegistry(key.get(), currentValue);
	systemValue = currentValue;
}

void AbstractRegCheckItem::cancel() {
	setCurrentValue(systemValue);
}

