// tuneapp.


#include "AbstractRegValueItem.h"

#include "../ConfigReader.h"
#include "../ui/ValueItemView.h"


AbstractRegValueItem::AbstractRegValueItem()
	: minValue(INT_MIN), maxValue(INT_MAX),
	valueSteps(1), blockSteps(10), scale(1), unit(NULL_STRING) {
}

void AbstractRegValueItem::setParams(xml::Node* params) throw(ConfigException) {
	for (	xml::Node::const_iterator children = params->begin();
			children != params->end();
			++children) {
		xml::Node* child = *children;
		if (child->getType() == xml::Node::ELEMENT) {
			const wstring& name = child->getName();
			if (name == L"min") {
				minValue = parseInt(child->getText());
			} else if (name == L"max") {
				maxValue = parseInt(child->getText());
			} else if (name == L"step") {
				valueSteps = parseInt(child->getText());
			} else if (name == L"block") {
				blockSteps = parseInt(child->getText());
			} else if (name == L"scale") {
				scale = parseInt(child->getText());
			} else if (name == L"unit") {
				unit = child->getText();
			}
		}
	}
	AbstractRegItem::setParams(params);

	init();
}

void AbstractRegValueItem::init() {
	defaultValue = parseInt(getDefaultValueString());
	const wstring& defaultMeanString = getDefaultMeanString();
	if (defaultMeanString == NULL_STRING) {
		defaultMean = defaultValue;
	} else {
		defaultMean = parseInt(defaultMeanString);
	}

	readRegistry();
}

void AbstractRegValueItem::readRegistry() {
	try {
		auto_ptr<RegKey> key(openRegKey(KEY_READ));
		systemValue = readRegistry(key.get());
	} catch (RegKeyException) {
		systemValue = defaultValue;
	}
	currentValue = systemValue;
	// make short help
}

awt::Component* AbstractRegValueItem::getViewComponent() {
	return new ValueItemView(this);
}

int AbstractRegValueItem::getSystemValue() const {
	return systemValue;
}

int AbstractRegValueItem::getCurrentValue() const {
	return currentValue;
}

int AbstractRegValueItem::getDefaultValue() const {
	return defaultValue;
}

void AbstractRegValueItem::setDefaultValue(int defaultValue) {
	if (this->defaultValue == defaultValue) return;
	this->defaultValue = defaultValue;
	firePropertyChange(PROP_DEFAULT_VALUE);
}

int AbstractRegValueItem::getDefaultMean() const {
	return defaultMean;
}

int AbstractRegValueItem::getMinimumValue() const {
	return minValue;
}

int AbstractRegValueItem::getMaximumValue() const {
	return maxValue;
}

int AbstractRegValueItem::getValueSteps() const {
	return valueSteps;
}

int AbstractRegValueItem::getBlockSteps() const {
	return blockSteps;
}

int AbstractRegValueItem::getScale() const {
	return scale;
}

const wstring& AbstractRegValueItem::getUnit() const {
	return unit;
}

void AbstractRegValueItem::setCurrentValue(int currentValue) {
	if (this->currentValue == currentValue) return;
	this->currentValue = currentValue;
	firePropertyChange(PROP_CURRENT_VALUE);
}

void AbstractRegValueItem::setDefault() {
	setCurrentValue(defaultValue);
}

void AbstractRegValueItem::apply() throw(RegKeyException) {
	if (currentValue == systemValue) return;

	writeRegistry();
	setChanged();
}

void AbstractRegValueItem::writeRegistry() throw(RegKeyException) {
	auto_ptr<RegKey> key(openRegKey(KEY_WRITE | KEY_READ));
	writeRegistry(key.get(), currentValue);
	systemValue = currentValue;
}

void AbstractRegValueItem::cancel() {
	setCurrentValue(systemValue);
}
