// tuneapp.


#include "PriorityControlItem.h"

#include "../ConfigReader.h"
#include "../utils.h"

#include "../awt/Label.h"


PriorityControlItem::PriorityControlItem()
		: server(false), lengthLabels(2), boostLabels(3) {
}

void PriorityControlItem::setParams(xml::Node* params) throw(ConfigException) {
	for (	xml::Node::const_iterator children = params->begin();
			children != params->end();
			++children) {
		xml::Node* child = *children;
		if (child->getType() == xml::Node::ELEMENT) {
			const wstring& name = child->getName();
			if (name == L"server") {
				server = true;
			} else if (name == L"select") {
				const wstring& id = child->getAttribute(L"id");
				if (id == L"length") {
					setLengthParams(child);
				} else if (id == L"boost") {
					setBoostParams(child);
				}
			}
		}
	}
	AbstractRegBitValueItem::setParams(params);
}

void PriorityControlItem::setLengthParams(xml::Node* params) {
	int i = 0;
	for (	xml::Node::const_iterator children = params->begin();
			children != params->end();
			++children) {
		xml::Node* child = *children;
		if (child->getType() == xml::Node::ELEMENT) {
			const wstring& name = child->getName();
			if (name == L"name") {
				lengthName = child->getText();
			} else if (name == L"item") {
				lengthLabels.at(i) = child->getText();
				i++;
			}
		}
	}
}

void PriorityControlItem::setBoostParams(xml::Node* params) {
	int i = 0;
	for (	xml::Node::const_iterator children = params->begin();
			children != params->end();
			++children) {
		xml::Node* child = *children;
		if (child->getType() == xml::Node::ELEMENT) {
			const wstring& name = child->getName();
			if (name == L"name") {
				boostName = child->getText();
			} else if (name == L"item") {
				boostLabels.at(i) = child->getText();
				i++;
			}
		}
	}
}

bool PriorityControlItem::isShortLength() const {
	int value = getCurrentValue();
	int lengthInt = (value & 0x30) >> 4;
	if ((lengthInt == 0) || (lengthInt == 3)) {
		// default length dependant product type
		if (server) {
			return false;
		} else {
			return true;
		}
	}
	if (lengthInt == 1) {
		return false;
	} else { // lengthInt == 2
		return true;
	}
}

void PriorityControlItem::setShortLength(bool shortLength) {
	int value = getCurrentValue();
	value = (value & 0x0f) | ((shortLength ? 2 : 1) << 4);
	setCurrentValue(value);
}

int PriorityControlItem::getBoost() const {
	int value = getCurrentValue();
	int variableInt = (value & 0x0c) >> 2;
	if ((variableInt == 0) || (variableInt == 3)) {
		// default variable or fix dependant product type
		if (server) {
			return 0;
		}
	} else if (variableInt == 2) {
		return 0;
	}
	int boost = (value & 0x03);
	if (boost == 3) boost = 2;
	return boost;
}

void PriorityControlItem::setBoost(int boost) {
	if ((boost < 0) || (boost > 2)) {
		throw IllegalArgumentException(L"boost: " + toString(boost));
	}
	int value = getCurrentValue();
	if (boost == 0) {
		// fix and 0
		value = (value & 0x30) | (2 << 2) | boost;
	} else {
		// variable and boost
		value = (value & 0x30) | (1 << 2) | boost;
	}
	setCurrentValue(value);
}

awt::Component* PriorityControlItem::getViewComponent() {
	return new View(this);
}


// View


PriorityControlItem::View::View(PriorityControlItem* target)
		: target(target), lengthCheckbox(2), boostCheckbox(3) {
	createComponents();
}

void PriorityControlItem::View::createComponents() {
	// awt::Panel* lengthControl = new awt::Panel();
	awt::CheckboxGroup* lengthGroup = new awt::CheckboxGroup();
	lengthGroup->setLayout(utils::leftFlowLayout);
	for (int i = 0; i < lengthCheckbox.size(); i++) {
		lengthCheckbox.at(i) = new awt::Checkbox(target->lengthLabels.at(i), false, lengthGroup);
		lengthGroup->add(lengthCheckbox.at(i));
	}

	// awt::Panel* boostControl = new awt::Panel();
	awt::CheckboxGroup* boostGroup = new awt::CheckboxGroup();
	boostGroup->setLayout(utils::leftFlowLayout);
	for (i = 0; i < boostCheckbox.size(); i++) {
		boostCheckbox.at(i) = new awt::Checkbox(target->boostLabels.at(i), false, boostGroup);
		boostGroup->add(boostCheckbox.at(i));
	}

	setLayout(utils::c2GridFlowLayout);
	add(new awt::Label(target->lengthName));
	add(lengthGroup);
	add(new awt::Label(target->boostName));
	add(boostGroup);

	if (target->isShortLength()) {
		lengthCheckbox.at(0)->setState(true);
	} else {
		lengthCheckbox.at(1)->setState(true);
	}
	boostCheckbox.at(target->getBoost())->setState(true);

	lengthCheckbox.at(0)->addItemListener(new LengthSetter(target, true));
	lengthCheckbox.at(1)->addItemListener(new LengthSetter(target, false));
	for (i = 0; i < boostCheckbox.size(); i++) {
		boostCheckbox.at(i)->addItemListener(new BoostSetter(target, i));
	}
}

PriorityControlItem::View::LengthSetter::LengthSetter(PriorityControlItem* target, bool shortLength)
		: target(target), shortLength(shortLength) {
}

void PriorityControlItem::View::LengthSetter::itemStateChanged(awt::ItemEvent& event) {
	target->setShortLength(shortLength);
}

PriorityControlItem::View::BoostSetter::BoostSetter(PriorityControlItem* target, int boost)
		: target(target), boost(boost) {
}

void PriorityControlItem::View::BoostSetter::itemStateChanged(awt::ItemEvent& event) {
	target->setBoost(boost);
}
