// tuneapp.


#include "RegCheckItem.h"

#include "../ConfigReader.h"


RegCheckItem::RegCheckItem()
		: trueString(NULL_STRING), falseString(NULL_STRING) {
}

void RegCheckItem::setParams(xml::Node* params) throw(ConfigException) {
	for (	xml::Node::const_iterator children = params->begin();
			children != params->end();
			++children) {
		xml::Node* child = *children;
		if (child->getType() == xml::Node::ELEMENT) {
			const wstring& name = child->getName();
			if (name == L"true") {
				trueString = child->getText();
			} else if (name == L"false") {
				falseString = child->getText();
			}
		}
	}
	AbstractRegCheckItem::setParams(params);

	init();
}

void RegCheckItem::init() {
	const wstring& defaultValueString = getDefaultValueString();
	if (defaultValueString == trueString) {
		defaultValue = true;
	} else if (defaultValueString == falseString) {
		defaultValue = false;
	} else {
		throw ConfigException(L"default no match: " + defaultValueString);
	}

	AbstractRegCheckItem::readRegistry();
}

bool RegCheckItem::readRegistry(RegKey* key) throw(RegKeyException) {
	const wstring defaultValueString = getDefaultValueString();
	switch (getRegType()) {
	case REG_DWORD:
		{
			int defaultValueInt = parseInt(defaultValueString);
			int trueInt = parseInt(trueString);
			int falseInt = parseInt(falseString);
			int systemValueInt = key->getIntValue(getRegValueName());
			if (systemValueInt == trueInt) {
				return true;
			} else if (systemValueInt == falseInt) {
				return false;
			} else {
				return defaultValue;
			}
		}
	case REG_BINARY:
		{
			int length;
			auto_array<BYTE> systemValueBytes(key->getBytesValue(getRegValueName(), &length));
			wstring systemValueString;
			bytesToHexString(systemValueBytes, length, &systemValueString);
			if (systemValueString == trueString) {
				return true;
			} else if (systemValueString == falseString) {
				return false;
			} else {
				return defaultValue;
			}
		}
	case REG_SZ:
	case REG_EXPAND_SZ:
	case REG_MULTI_SZ:
		{
			const wstring& systemValueString = key->getStringValue(getRegValueName());
			if (systemValueString == trueString) {
				return true;
			} else if (systemValueString == falseString) {
				return false;
			} else {
				return defaultValue;
			}
		}
	default:
		throw ConfigException(L"RegType");
	}
}

void RegCheckItem::writeRegistry(RegKey* key, bool value) throw(RegKeyException) {
	if ((value == defaultValue) && isDeleteForDefault()) {
		key->deleteValue(getRegValueName());
	} else {
		const wstring& valueString = (value ? trueString : falseString);
		int type = getRegType();
		switch (type) {
		case REG_DWORD:
			{
				int valueInt = parseInt(valueString);
				key->setValue(getRegValueName(), valueInt);
				break;
			}
		case REG_BINARY:
			{
				int length;
				auto_array<BYTE> bytes(hexStringToBytes(valueString, &length));
				key->setValue(getRegValueName(), bytes, length);
				break;
			}
		case REG_SZ:
		case REG_EXPAND_SZ:
		case REG_MULTI_SZ:
			key->setValue(getRegValueName(), type, valueString);
			break;
		default:
			throw ConfigException(L"RegType");
		}
	}
}

void RegCheckItem::store(RegistryStorage* storage) {
	RegistryStorage::Key* key = storage->getKey(getRegKeyName());
	if ((currentValue == defaultValue) && isDeleteForDefault()) {
		key->setDeleteValue(getRegValueName());
	} else {
		const wstring& valueString = (currentValue ? trueString : falseString);
		int type = getRegType();
		switch (type) {
		case REG_DWORD:
			{
				int valueInt = parseInt(valueString);
				key->setDWORD(getRegValueName(), valueInt);
				break;
			}
		case REG_BINARY:
			{
				int length;
				auto_array<BYTE> valueBytes(hexStringToBytes(valueString, &length));
				key->setBINARY(getRegValueName(), valueBytes, length);
				break;
			}
		case REG_SZ:
		case REG_EXPAND_SZ:
		case REG_MULTI_SZ:
			key->setString(getRegValueName(), type, valueString);
			break;
		default:
			throw ConfigException(L"RegType");
		}
	}
}
