// tuneapp.


#ifndef __REG_KEY_H__
#define __REG_KEY_H__


#include "RegKeyException.h"

#include <string>
#include <vector>
#include <map>
using namespace std;

#include <windows.h>


class RegKey {
public:
	static HKEY getRootID(const wstring& rootString);
	static bool isStringType(DWORD type);

	static void deleteKeyIfExists(const wstring& keyName) throw(RegKeyException);
	static void deleteValueIfExists(const wstring& keyName, const wstring& valueName) throw(RegKeyException);

private:
	HKEY handle;

	void RegKey::createHandle(HKEY rootID, const wstring& subKey, REGSAM access) throw(RegKeyException);

public:
	RegKey(const wstring& keyName, REGSAM access) throw(RegKeyException);
	RegKey(HKEY rootID, const wstring& subKey, REGSAM access) throw(RegKeyException);
	virtual ~RegKey();

	void flush() throw(RegKeyException);
	bool hasValue(const wstring& valueName);
	int getIntValue(const wstring& valueName) throw(RegKeyException);
	int getIntValue(const wstring& valueName, int defaultValue);
	BYTE* getBytesValue(const wstring& valueName, int* length) throw(RegKeyException);
	wstring getStringValue(const wstring& valueName) throw(RegKeyException);
	wstring getStringValue(const wstring& valueName, const wstring& defaultValue) throw(RegKeyException);
	void setValue(const wstring& valueName, int value) throw(RegKeyException);
	void setValue(const wstring& valueName, BYTE* value, int length);
	void setValue(const wstring& valueName, DWORD type, const wstring& value) throw(RegKeyException);
	void deleteSubKey(const wstring& keyName) throw(RegKeyException);
	void deleteValue(const wstring& valueName) throw(RegKeyException);
	vector<wstring>* getSubKeys(vector<wstring>* subKeys) throw(RegKeyException);
	map<wstring, DWORD>* getValueTypes(map<wstring, DWORD>* valueTypes) throw(RegKeyException);
};


#endif // __REG_KEY_H__
