// tuneapp.


#include "RegMultiListValueItem.h"

#include "../ConfigReader.h"


RegMultiListValueItem::RegMultiListValueItem()
		: entryIndex(0) {
}

void RegMultiListValueItem::setParams(xml::Node* params) throw(ConfigException) {
	for (	xml::Node::const_iterator children = params->begin();
			children != params->end();
			++children) {
		xml::Node* child = *children;
		if (child->getType() == xml::Node::ELEMENT) {
			const wstring& name = child->getName();
			if (name == L"index") {
				entryIndex = parseInt(child->getText());
			}
		}
	}
	RegMultiValueItem::setParams(params);
}

int RegMultiListValueItem::readRegistry(RegKey* regKey) throw(RegKeyException) {
	wstring systemValueString = regKey->getStringValue(getRegValueName(), NULL_STRING);
	if (systemValueString == NULL_STRING) {
		return getDefaultValue();
	} else {
		map<wstring, wstring> valueMap;
		parseValueMap(systemValueString, &valueMap);
		map<wstring, wstring>::const_iterator found
				= valueMap.find(getRegValueEntryName());
		if (found == valueMap.end()) {
			return getDefaultValue();
		} else {
			vector<wstring> valueList;
			parseValueList((*found).second, &valueList);
			if (entryIndex >= valueList.size()) {
				return getDefaultValue();
			} else {
				try {
					return parseInt(valueList[entryIndex]);
				} catch (NumberFormatException) {
					// showRegistryIncorrectedDialog(systemValueString);
					return getDefaultValue();
				}
			}
		}
	}
	// make short help
}

vector<wstring>* RegMultiListValueItem::parseValueList(
			const wstring& string, vector<wstring>* valueList) {
	wstring::size_type comma;
	for (wstring::size_type i = 0; i < string.length(); i = comma + 1) {
		comma = string.find(',', i);
		if (comma != wstring::npos) {
			valueList->push_back(string.substr(i, comma - i));
		} else {
			valueList->push_back(string.substr(i));
			break;
		}
	}
	return valueList;
}

void RegMultiListValueItem::writeRegistry(RegKey* key, int value) throw(RegKeyException) {
	wstring systemValueString = key->getStringValue(getRegValueName(), NULL_STRING);
	wstring valueString;
	try {
		valueString = createValueString(systemValueString, value);
	} catch (IllegalArgumentException) {
		valueString = createValueString(getCorrectiveValueString(), value);
	}
	key->setValue(getRegValueName(), getRegType(), valueString);
}

static wstring toString(const vector<wstring>& list) {
	wstring buffer;
	for (int i = 0; i < list.size(); i++) {
		if (i > 0) {
			buffer += L',';
		}
		buffer += list[i];
	}
	return buffer;
}

wstring RegMultiListValueItem::createValueString(const wstring& systemValueString, int value) const
		throw(IllegalArgumentException) {
	if (systemValueString == NULL_STRING) {
		throw IllegalArgumentException(L"no systemValueString");
	}
	const wstring& entryName = getRegValueEntryName();
	wstring::size_type start;
	if (startsWith(systemValueString, entryName + L"=")) {
		start = 0;
	} else {
		start = systemValueString.find(L" " + entryName + L"=");
		if (start == wstring::npos) {
			throw IllegalArgumentException(L"no entryName");
		}
		start++;
	}
	wstring::size_type equal = systemValueString.find('=', start);
	wstring::size_type nextStart = systemValueString.find(' ', start);
	wstring valueListString;
	if (nextStart == wstring::npos) {
		valueListString = systemValueString.substr(equal + 1);
	} else {
		valueListString = systemValueString.substr(equal + 1, nextStart - (equal + 1));
		nextStart++;
	}
	vector<wstring> valueList;
	parseValueList(valueListString, &valueList);
	if (entryIndex >= valueList.size()) {
		throw IllegalArgumentException(L"index out of range");
	}
	valueList[entryIndex] = toString(value);
	valueListString = toString(valueList);
	wstring valueString
			= (start == 0 ? L"" : systemValueString.substr(0, start))
			+ entryName + L"=" + valueListString
			+ (nextStart == wstring::npos ? L"" : L" " + systemValueString.substr(nextStart));
	return valueString;
}

void RegMultiListValueItem::store(RegistryStorage* storage) {
	RegistryStorage::Key* key = storage->getKey(getRegKeyName());
	wstring systemValueString = key->getValueString(getRegValueName());
	if (systemValueString == NULL_STRING) {
		auto_ptr<RegKey> regKey(openRegKey(KEY_READ));
		try {
			systemValueString = regKey->getStringValue(getRegValueName());
		} catch (RegKeyException e) {
			throw;
		}
	}
	wstring valueString;
	try {
		valueString = createValueString(
				systemValueString, getCurrentValue());
	} catch (IllegalArgumentException) {
		valueString = createValueString(
				getCorrectiveValueString(), getCurrentValue());
	}
	key->setString(getRegValueName(), getRegType(), valueString);
}
