// tuneapp.


#include "RegMultiStringCheckItem.h"

#include "../ConfigReader.h"


const wstring RegMultiStringCheckItem::MULTI_SZ_TERM = wstring(1, '\0');


RegMultiStringCheckItem::RegMultiStringCheckItem()
		: checkPattern(NULL_STRING) {
}

void RegMultiStringCheckItem::setParams(xml::Node* params) throw(ConfigException) {
	for (	xml::Node::const_iterator children = params->begin();
			children != params->end();
			++children) {
		xml::Node* child = *children;
		if (child->getType() == xml::Node::ELEMENT) {
			const wstring& name = child->getName();
			if (name == L"pattern") {
				checkPattern = child->getText();
			} else if (name == L"default") {
				defaultValue = parseBool(child->getText());
			}
		}
	}
	AbstractRegCheckItem::setParams(params);

	init();
}

void RegMultiStringCheckItem::init() {
	AbstractRegCheckItem::readRegistry();
}

bool RegMultiStringCheckItem::readRegistry(RegKey* key) throw(RegKeyException) {
	const wstring& systemValueString = key->getStringValue(getRegValueName());
	wstring::size_type found = findPatternMatch(systemValueString);
	if (found == wstring::npos) {
		return false;
	} else {
		return true;
	}
}

void RegMultiStringCheckItem::writeRegistry(RegKey* key, bool value) throw(RegKeyException) {
	const wstring& systemValueString = key->getStringValue(getRegValueName(), MULTI_SZ_TERM);
	wstring valueString = createValueString(systemValueString, value);
	key->setValue(getRegValueName(), REG_MULTI_SZ, valueString);
}

void RegMultiStringCheckItem::store(RegistryStorage* storage) {
	RegistryStorage::Key* key = storage->getKey(getRegKeyName());
	wstring systemValueString = key->getValueString(getRegValueName());
	if (systemValueString == NULL_STRING) {
		auto_ptr<RegKey> key(openRegKey(KEY_READ));
		systemValueString = key->getStringValue(getRegValueName(), MULTI_SZ_TERM);
	}
	wstring valueString = createValueString(systemValueString, getCurrentValue());
	key->setMULTI_SZ(getRegValueName(), valueString);
}

wstring::size_type RegMultiStringCheckItem::findPatternMatch(const wstring& systemValueString) {
	return toLowerCase(systemValueString).find(toLowerCase(checkPattern) + MULTI_SZ_TERM);
}

wstring RegMultiStringCheckItem::createValueString(const wstring& systemValueString, bool value) {
	wstring::size_type found = findPatternMatch(systemValueString);
	if (found == wstring::npos) {
		if (value) {
			return systemValueString + checkPattern + MULTI_SZ_TERM;
		} else {
			return systemValueString;
		}
	} else {
		if (value) {
			return systemValueString;
		} else {
/*
			wstring valueString = systemValueString;
			valueString.erase(found, checkPattern.length() + 1);
*/
			wstring valueString;
			valueString.append(systemValueString, 0, found);
			int endPoint = found + checkPattern.length() + 1;
			valueString.append(systemValueString, endPoint, systemValueString.length() - endPoint);
			return valueString;
		}
	}
}
