// tuneapp.


#include "RegMultiValueItem.h"

#include "../ConfigReader.h"


RegMultiValueItem::RegMultiValueItem()
	: entryName(NULL_STRING), correctiveValueString(NULL_STRING) {
}

void RegMultiValueItem::setParams(xml::Node* params) throw(ConfigException) {
	for (	xml::Node::const_iterator children = params->begin();
			children != params->end();
			++children) {
		xml::Node* child = *children;
		if (child->getType() == xml::Node::ELEMENT) {
			const wstring& name = child->getName();
			if (name == L"name") {
				entryName = child->getText();
			} else if (name == L"correction") {
				correctiveValueString = child->getText();
			}
		}
	}
	AbstractRegValueItem::setParams(params);
}

const wstring& RegMultiValueItem::getRegValueEntryName() const {
	return entryName;
}

const wstring& RegMultiValueItem::getCorrectiveValueString() const {
	return correctiveValueString;
}

int RegMultiValueItem::readRegistry(RegKey* key) throw(RegKeyException) {
	const wstring& systemValueString = key->getStringValue(getRegValueName(), NULL_STRING);
	if (systemValueString == NULL_STRING) {
		return getDefaultValue();
	} else {
		map<wstring, wstring> valueMap;
		parseValueMap(systemValueString, &valueMap);
		map<wstring, wstring>::const_iterator found = valueMap.find(entryName);
		if (found == valueMap.end()) {
			return getDefaultValue();
		} else {
			try {
				return parseInt((*found).second);
			} catch (NumberFormatException) {
				// showRegistryIncorrectedDialog(systemValueString);
				return getDefaultValue();
			}
		}
	}
	// make short help
}

map<wstring, wstring>* RegMultiValueItem::parseValueMap(const wstring& string, map<wstring, wstring>* valueMap) {
	wstring::size_type space;
	for (wstring::size_type i = 0; i < string.length(); i = space + 1) {
		space = string.find(' ', i);
		if (space == wstring::npos) space = string.length();
		wstring token = string.substr(i, space - i);
		wstring::size_type equal = token.find('=');
		wstring name, value;
		if (equal == wstring::npos) {
			name = token;
			// value = "";
		} else {
			name = token.substr(0, equal);
			value = token.substr(equal + 1);
		}
		(*valueMap)[name] = value;
	}
	return valueMap;
}

void RegMultiValueItem::writeRegistry(RegKey* key, int value) throw(RegKeyException) {
	wstring systemValueString = key->getStringValue(getRegValueName(), NULL_STRING);
	wstring valueString = createValueString(systemValueString, value);
	key->setValue(getRegValueName(), getRegType(), valueString);
}

wstring RegMultiValueItem::createValueString(const wstring& _svs, int value) const {
	wstring systemValueString = _svs;
	if (systemValueString == NULL_STRING) {
		systemValueString = correctiveValueString;
	}
	wstring valueString;
	wstring::size_type start;
	wstring::size_type nextStart;
	if (startsWith(systemValueString, entryName + L"=")) {
		start = 0;
		nextStart = systemValueString.find(' ');
		if (nextStart != wstring::npos) nextStart++;
	} else {
		start = systemValueString.find(L" " + entryName + L"=");
		if (start == wstring::npos) {
			start = systemValueString.length();
			nextStart = wstring::npos;
		} else {
			start++;
			nextStart = systemValueString.find(' ', start);
			if (nextStart != wstring::npos) nextStart++;
		}
	}
	valueString
			= (start == 0 ? L"" : systemValueString.substr(0, start))
			+ entryName + L"=" + toString(value)
			+ (nextStart == wstring::npos ? L"" : L" " + systemValueString.substr(nextStart));
	return valueString;
}

void RegMultiValueItem::store(RegistryStorage* storage) {
	RegistryStorage::Key* key = storage->getKey(getRegKeyName());
	wstring systemValueString = key->getValueString(getRegValueName());
	if (systemValueString == NULL_STRING) {
		try {
			auto_ptr<RegKey> regKey(openRegKey(KEY_READ));
			systemValueString
						= regKey->getStringValue(getRegValueName(), NULL_STRING);
		} catch (RegKeyException e) {
			throw;
		}
	}
	wstring valueString
			= createValueString(systemValueString, getCurrentValue());
	key->setString(getRegValueName(), getRegType(), valueString);
}
