// tuneapp.


#include "RegValueItem.h"

#include "../lang.h"


int RegValueItem::readRegistry(RegKey* key) throw(RegKeyException) {
	switch (getRegType()) {
	case REG_DWORD:
		return key->getIntValue(getRegValueName(), getDefaultValue());
	case REG_SZ:
	case REG_EXPAND_SZ:
	case REG_MULTI_SZ:
		{
			wstring systemValueString
					= key->getStringValue(getRegValueName(), NULL_STRING);
			if (systemValueString == NULL_STRING) {
				return getDefaultValue();
			} else {
				try {
					return parseInt(systemValueString);
				} catch (NumberFormatException e) {
					// showRegistryIncorrectedDialog(systemValueString);
					return getDefaultValue();
				}
			}
		}
	default:
		throw ConfigException(L"RegType");
	}
}

void RegValueItem::writeRegistry(RegKey* key, int value) throw(RegKeyException) {
	if ((value == getDefaultValue()) && isDeleteForDefault()) {
		key->deleteValue(getRegValueName());
	} else {
		int type = getRegType();
		switch (type) {
		case REG_DWORD:
			key->setValue(getRegValueName(), value);
			break;
		case REG_SZ:
		case REG_EXPAND_SZ:
		case REG_MULTI_SZ:
			key->setValue(getRegValueName(), type, toString(value));
			break;
		default:
			throw ConfigException(L"RegType");
		}
	}
}

void RegValueItem::store(RegistryStorage* storage) {
	int currentValue = getCurrentValue();
	int defaultValue = getDefaultValue();
	RegistryStorage::Key* key = storage->getKey(getRegKeyName());
	if ((currentValue == defaultValue) && isDeleteForDefault()) {
		key->setDeleteValue(getRegValueName());
	} else {
		int type = getRegType();
		switch (type) {
		case REG_DWORD:
			key->setDWORD(getRegValueName(), currentValue);
			break;
		case REG_SZ:
		case REG_EXPAND_SZ:
		case REG_MULTI_SZ:
			key->setString(getRegValueName(), type, toString(currentValue));
			break;
		default:
			throw ConfigException(L"RegType");
		}
	}
}
