// tuneapp.


#ifndef __REGISTRY_SOTRAGE_H__
#define __REGISTRY_SOTRAGE_H__


#pragma warning(disable:4786)


#include "RegKey.h"

#include <string>
#include <map>
#include <vector>
#include <ostream>
using namespace std;

#include <windows.h>


class RegistryStorage {
private:
	class Value {
	protected:
		const wstring name;
	public:
		Value(const wstring& name);
		virtual ~Value();
		virtual void write(wostream& out) = 0;
		virtual wstring toString() = 0;
	protected:
		int writeNameEqual(wostream& out);
	};

	class DWORDValue : public Value {
	private:
		DWORD value;
	public:
		DWORDValue(const wstring& name, DWORD value);
		virtual void write(wostream& out);
		virtual wstring toString();
	};

	class AbstractHexValue : public Value {
	private:
		DWORD type;
		BYTE* value;
		int length;
	protected:
		AbstractHexValue(const wstring& name, DWORD type, const BYTE* value, int length);
	public:
		virtual ~AbstractHexValue();
		virtual void write(wostream& out);
		virtual wstring toString();
	protected:
		virtual int writeType(wostream& out);
	};

	class BINARYValue : public AbstractHexValue {
	public:
		BINARYValue(const wstring& name, const BYTE* value, int length);
	protected:
		virtual int writeType(wostream& out);
	};

	class SZValue : public Value {
	private:
		const wstring value;
	public:
		SZValue(const wstring& name, const wstring& value);
	public:
		virtual void write(wostream& out);
		virtual wstring toString();
	};

	class HexStringValue : public AbstractHexValue {
	private:
		const wstring value;
	protected:
		HexStringValue(const wstring& name, DWORD type, const wstring& value);
		virtual wstring toString();
	private:
		static void toBytes(const wstring& string, vector<signed char>* bytes);
	};

	class EXPAND_SZValue : public HexStringValue {
	public:
		EXPAND_SZValue(const wstring& name, const wstring& value);
	};

	class MULTI_SZValue : public HexStringValue {
	public:
		MULTI_SZValue(const wstring& name, const wstring& value);
	};

	class DeleteValue : public Value {
	public:
		DeleteValue(const wstring& name);
		void write(wostream& out);
		virtual wstring toString();
	};

public:
	class Key {
	private:
		RegistryStorage* storage;
		const wstring keyName;
		bool deleteAtFirst;
		map<wstring, Value*> valueMap;
	public:
		Key(RegistryStorage* storage, const wstring& keyName);
		~Key();
	private:
		void putValue(const wstring& valueName, Value* value);
	public:
		void setDeleteAtFirst();
		void setDWORD(const wstring& valueName, DWORD value);
		void setBINARY(const wstring& valueName, const BYTE* value, int length);
		void setSZ(const wstring& valueName, const wstring& value);
		void setEXPAND_SZ(const wstring& valueName, const wstring& value);
		void setMULTI_SZ(const wstring& valueName, const wstring& value);
		void setString(const wstring& valueName, DWORD type, const wstring& value);
		void setDeleteValue(const wstring& valueName);
		wstring getValueString(const wstring& valueName);
		void exportCurrentAll() throw(RegKeyException);
		void exportCurrentValues() throw(RegKeyException);

		virtual void write(wostream& out);
		void writeDelete(wostream& out);
	};

private:
	class DeleteKey : public Key {
	public:
		DeleteKey(RegistryStorage* storage, const wstring& key);

		virtual void write(wostream& out);
	};

private:
	const wstring baseName;
	map<wstring, Key*> keyMap;

public:
	RegistryStorage(const wstring& baseName);
	~RegistryStorage();
	void save();
	Key* getKey(const wstring& keyName);
	void deleteKey(const wstring& keyName);
	void cancelExport(const wstring& keyName);

private:
	void write(wostream& out);
};


#endif // __REGISTRY_SOTRAGE_H__
