// tuneapp.


#include "NICCheckItem.h"


NICCheckItem::NICCheckItem()
		: interfacesParams(NULL),
		selectionType(NULL_STRING), inheritor(false), inheritee(NULL), currentInherit(false) {
}

void NICCheckItem::setParams(xml::Node* params) throw(ConfigException) {
	for (	xml::Node::const_iterator children = params->begin();
			children != params->end();
			++children) {
		xml::Node* child = *children;
		if (child->getType() == xml::Node::ELEMENT) {
			const wstring& name = child->getName();
			if (name == L"select") {
				selectionType = child->getText();
			} else if (name == L"interfaces") {
				interfacesParams = child;
			} else if (name == L"inherit") {
				setInheritor();
			}
		}
	}
	RegCheckItem::setParams(params);
}

bool NICCheckItem::readRegistry(RegKey* key) throw(RegKeyException) {
	if (inheritor) {
		systemInherit = !key->hasValue(getRegValueName());
		currentInherit = systemInherit;
		inheritDefaultValue();
	}
	// make short help
	return RegCheckItem::readRegistry(key);
}

Item* NICCheckItem::cloneForInterface(const wstring& regKey) throw(ConfigException) {
	NICCheckItem* clone = new NICCheckItem(*this);
	clone->setRegKeyName(regKey);
	clone->inheritee = this;
	if (interfacesParams != NULL) {
		clone->setParams(interfacesParams);
	} else {
		clone->AbstractRegCheckItem::readRegistry();
	}
	return clone;
}

const wstring& NICCheckItem::getSelectionType() const {
	return selectionType;
}

bool NICCheckItem::isInheritor() const {
	return inheritor;
}

void NICCheckItem::setInheritor() {
	inheritor = true;
	setDeleteForDefault(false);
	inheritee->addPropertyChangeListener(new InheriteeListener(this));
}

void NICCheckItem::InheriteeListener::propertyChange(PropertyChangeEvent& event) {
	const wstring& name = event.getPropertyName();
	if (name == PROP_CURRENT_VALUE) {
		target->inheritDefaultValue();
	}
}

Item* NICCheckItem::getInheritee() const {
	return inheritee;
}

bool NICCheckItem::isInherited() const {
	return currentInherit;
}

void NICCheckItem::setInherited(bool currentInherit) {
	if (this->currentInherit == currentInherit) return;
	this->currentInherit = currentInherit;
	if (currentInherit) {
		inheritDefaultValue();
	}
}

void NICCheckItem::setDefault() {
	if (inheritor) {
		setInherited(true);
	}
	RegCheckItem::setDefault();
}

void NICCheckItem::apply() throw(RegKeyException) {
	if (inheritor) {
		if (currentInherit) {
			if (systemInherit) return;
			deleteRegistry();
			setChanged();
		} else {
			if (systemInherit) {
				AbstractRegCheckItem::writeRegistry();
				setChanged();
			} else {
				RegCheckItem::apply();
			}
		}
	} else {
		RegCheckItem::apply();
	}
}

void NICCheckItem::writeRegistry(RegKey* key, bool value)
		throw(RegKeyException) {
	RegCheckItem::writeRegistry(key, value);
	systemInherit = false;
}

void NICCheckItem::deleteRegistry() throw(RegKeyException) {
	RegKey::deleteValueIfExists(getRegKeyName(), getRegValueName());
	systemInherit = true;
}

void NICCheckItem::cancel() {
	if (inheritor) {
		setInherited(systemInherit);
	}
	RegCheckItem::cancel();
}

void NICCheckItem::inheritDefaultValue() {
	bool defaultValue = inheritee->getCurrentValue();
	setDefaultValue(defaultValue);
	if (currentInherit) {
		setCurrentValue(defaultValue);
	}
}

void NICCheckItem::store(RegistryStorage* storage) {
	if (inheritor) {
		if (currentInherit) {
			RegistryStorage::Key* key = storage->getKey(getRegKeyName());
			key->setDeleteValue(getRegValueName());
		} else {
			RegCheckItem::store(storage);
		}
	} else {
		RegCheckItem::store(storage);
	}
}
