// tuneapp.


#ifndef __NIC_CHECK_ITEM_H__
#define __NIC_CHECK_ITEM_H__


#include "../RegCheckItem.h"
#include "NICParameterItem.h"


class NICCheckItem : public virtual RegCheckItem, public virtual NICParameterItem {
private:
	wstring selectionType;

	xml::Node* interfacesParams;

	bool inheritor;
	NICCheckItem* inheritee;
	bool systemInherit;
	bool currentInherit;

public:
	NICCheckItem();
	virtual void setParams(xml::Node* params) throw(ConfigException);
public:
	virtual Item* cloneForInterface(const wstring& regKey) throw(ConfigException);
	virtual const wstring& getSelectionType() const;

	virtual bool isInheritor() const;
	virtual Item* getInheritee() const;
	virtual bool isInherited() const;
	virtual void setInherited(bool currentInherit);

	virtual void setDefault();
	virtual void apply() throw(RegKeyException);
	virtual void cancel();

protected:
	virtual void setInheritor();
	virtual bool readRegistry(RegKey* key) throw(RegKeyException);
	using RegCheckItem::readRegistry;
	virtual void writeRegistry(RegKey* key, bool value);
	using RegCheckItem::writeRegistry;
	virtual void deleteRegistry() throw(RegKeyException);
	virtual void inheritDefaultValue();

	virtual void store(RegistryStorage* storage);

private:
	class InheriteeListener : public PropertyChangeListener {
	private:
		NICCheckItem* const target;
	public:
		InheriteeListener(NICCheckItem* target) : target(target) {}
		void propertyChange(PropertyChangeEvent& event);
	};
	friend class InheriteeListener;
};


#endif // __NIC_CHECK_ITEM_H__
