// tuneapp.


#include "NICValueItem.h"


NICValueItem::NICValueItem()
		: interfacesParams(NULL),
		selectionType(NULL_STRING), inheritor(false), inheritee(NULL), currentInherit(false) {
}

void NICValueItem::setParams(xml::Node* params) throw(ConfigException) {
	for (	xml::Node::const_iterator children = params->begin();
			children != params->end();
			++children) {
		xml::Node* child = *children;
		if (child->getType() == xml::Node::ELEMENT) {
			const wstring& name = child->getName();
			if (name == L"select") {
				selectionType = child->getText();
			} else if (name == L"interfaces") {
				interfacesParams = child;
			} else if (name == L"inherit") {
				setInheritor();
			}
		}
	}
	RegValueItem::setParams(params);
}

int NICValueItem::readRegistry(RegKey* key) throw(RegKeyException) {
	if (inheritor) {
		systemInherit = !key->hasValue(getRegValueName());
		currentInherit = systemInherit;
		inheritDefaultValue();
	}
	// make short help
	return RegValueItem::readRegistry(key);
}

Item* NICValueItem::cloneForInterface(const wstring& regKey) throw(ConfigException) {
	NICValueItem* clone = new NICValueItem(*this);
	clone->setRegKeyName(regKey);
	clone->inheritee = this;
	if (interfacesParams != NULL) {
		clone->setParams(interfacesParams);
	} else {
		clone->AbstractRegValueItem::readRegistry();
	}
	return clone;
}

const wstring& NICValueItem::getSelectionType() const {
	return selectionType;
}

bool NICValueItem::isInheritor() const {
	return inheritor;
}

void NICValueItem::setInheritor() {
	inheritor = true;
	setDeleteForDefault(false);
	inheritee->addPropertyChangeListener(new InheriteeListener(this));
}

void NICValueItem::InheriteeListener::propertyChange(PropertyChangeEvent& event) {
	const wstring& name = event.getPropertyName();
	if (name == PROP_CURRENT_VALUE) {
		target->inheritDefaultValue();
	}
}

Item* NICValueItem::getInheritee() const {
	return inheritee;
}

bool NICValueItem::isInherited() const {
	return currentInherit;
}

void NICValueItem::setInherited(bool currentInherit) {
	if (this->currentInherit = currentInherit) return;
	this->currentInherit = currentInherit;
	if (currentInherit) {
		inheritDefaultValue();
	}
}

void NICValueItem::setDefault() {
	if (inheritor) {
		setInherited(true);
	}
	RegValueItem::setDefault();
}

void NICValueItem::apply() throw(RegKeyException) {
	if (inheritor) {
		if (currentInherit) {
			if (systemInherit) return;
			deleteRegistry();
			setChanged();
		} else {
			if (systemInherit) {
				AbstractRegValueItem::writeRegistry();
				setChanged();
			} else {
				RegValueItem::apply();
			}
		}
	} else {
		RegValueItem::apply();
	}
}

void NICValueItem::writeRegistry(RegKey* key, int value) throw(RegKeyException) {
	RegValueItem::writeRegistry(key, value);
	systemInherit = false;
}

void NICValueItem::deleteRegistry() throw(RegKeyException) {
	RegKey::deleteValueIfExists(getRegKeyName(), getRegValueName());
	systemInherit = true;
}

void NICValueItem::cancel() {
	if (inheritor) {
		setInherited(systemInherit);
	}
	RegValueItem::cancel();
}

void NICValueItem::inheritDefaultValue() {
	int defaultValue = inheritee->getCurrentValue();
	setDefaultValue(defaultValue);
	if (currentInherit) {
		setCurrentValue(defaultValue);
	}
}

void NICValueItem::store(RegistryStorage* storage) {
	if (inheritor) {
		if (currentInherit) {
			RegistryStorage::Key* key = storage->getKey(getRegKeyName());
			key->setDeleteValue(getRegValueName());
		} else {
			RegValueItem::store(storage);
		}
	} else {
		RegValueItem::store(storage);
	}
}
