// tuneapp.


#ifndef __NIC_VALUE_ITEM_H__
#define __NIC_VALUE_ITEM_H__


#include "../RegValueItem.h"
#include "NICParameterItem.h"


class NICValueItem : public virtual RegValueItem, public virtual NICParameterItem {
private:
	wstring selectionType;

	xml::Node* interfacesParams;

	bool inheritor;
	NICValueItem* inheritee;
	bool systemInherit;
	bool currentInherit;

public:
	NICValueItem();
	virtual void setParams(xml::Node* params) throw(ConfigException);

	virtual Item* cloneForInterface(const wstring& regKey) throw(ConfigException);
	virtual const wstring& getSelectionType() const;

	virtual Item* getInheritee() const;
	virtual bool isInherited() const;
	void setInherited(bool currentInherit);
	virtual bool isInheritor() const;

	virtual void setDefault();
	virtual void apply() throw(RegKeyException);
	virtual void cancel();

protected:
	void setInheritor();
	virtual int readRegistry(RegKey* key) throw(RegKeyException);
	virtual void writeRegistry(RegKey* key, int value) throw(RegKeyException);
	virtual void deleteRegistry() throw(RegKeyException);
	void inheritDefaultValue();

	virtual void store(RegistryStorage* storage);

private:
	class InheriteeListener : public PropertyChangeListener {
	private:
		NICValueItem* const target;
	public:
		InheriteeListener(NICValueItem* target) : target(target) {}
		void propertyChange(PropertyChangeEvent& event);
	};
	friend class InheriteeListener;
};


#endif // __NIC_VALUE_ITEM_H__
