// tuneapp.


#include "RFC1323OptsItem.h"
#include "../../ConfigReader.h"
#include "../../utils.h"


RFC1323OptsItem::RFC1323OptsItem() {
}

void RFC1323OptsItem::setParams(xml::Node* params) throw(ConfigException) {
	for (	xml::Node::const_iterator children = params->begin();
			children != params->end();
			++children) {
		xml::Node* child = *children;
		if (ConfigReader::isElementName(child, L"select")) {
			const wstring& id = child->getAttribute(L"id");
			if (id == L"WindowScaling") {
				setWindowSaclingParams(child);
			} else if (id == L"Timestamping") {
				setTimestampingParams(child);
			}
		}
	}
	// make short help
	AbstractRegBitValueItem::setParams(params);
}

void RFC1323OptsItem::setWindowSaclingParams(xml::Node* params) {
	for (	xml::Node::const_iterator children = params->begin();
			children != params->end();
			++children) {
		xml::Node* child = *children;
		if (child->getType() == xml::Node::ELEMENT) {
			const wstring& name = child->getName();
			if (name == L"name") {
				windowScalingLabel = child->getText();
			}
		}
	}
}

void RFC1323OptsItem::setTimestampingParams(xml::Node* params) {
	for (	xml::Node::const_iterator children = params->begin();
			children != params->end();
			++children) {
		xml::Node* child = *children;
		if (child->getType() == xml::Node::ELEMENT) {
			const wstring& name = child->getName();
			if (name == L"name") {
				timestampingLabel = child->getText();
			}
		}
	}
}

static bool isWindowScalingEnabled(int value) {
	return ((value & 0x01) != 0);
}

static int setWindowScalingEnabled(int value, bool enabled) {
	if (enabled) {
		return (value | 0x01);
	} else {
		return (value & ~0x01);
	}
}

static bool isTimestampingEnabled(int value) {
	return ((value & 0x02) != 0);
}

static int setTimestampingEnabled(int value, boolean enabled) {
	if (enabled) {
		return (value | 0x02);
	} else {
		return (value & ~0x02);
	}
}

bool RFC1323OptsItem::isWindowScalingEnabled() {
	return ::isWindowScalingEnabled(getCurrentValue());
}

void RFC1323OptsItem::setWindowScalingEnabled(bool enabled) {
	setCurrentValue(::setWindowScalingEnabled(getCurrentValue(), enabled));
}

bool RFC1323OptsItem::isTimestampingEnabled() {
	return ::isTimestampingEnabled(getCurrentValue());
}

void RFC1323OptsItem::setTimestampingEnabled(bool enabled) {
	setCurrentValue(::setTimestampingEnabled(getCurrentValue(), enabled));
}

awt::Component* RFC1323OptsItem::getViewComponent() {
	return new View(this);
}


// View.

RFC1323OptsItem::View::View(RFC1323OptsItem* target) : target(target) {
	createComponents();
}

void RFC1323OptsItem::View::createComponents() {
	windowScalingCheckbox = new awt::Checkbox(
			target->windowScalingLabel, target->isWindowScalingEnabled());
	timestampingCheckbox = new awt::Checkbox(
			target->timestampingLabel, target->isTimestampingEnabled());

	setLayout(utils::c2GridFlowLayout);
	add(windowScalingCheckbox);
	add(timestampingCheckbox);

	windowScalingCheckbox->addItemListener(new WindowScalingListener(this));
	timestampingCheckbox->addItemListener(new TimestampingListener(this));
}

void RFC1323OptsItem::View::WindowScalingListener::itemStateChanged(awt::ItemEvent& event) {
	view->target->setWindowScalingEnabled(view->windowScalingCheckbox->getState());
}

void RFC1323OptsItem::View::TimestampingListener::itemStateChanged(awt::ItemEvent& event) {
	view->target->setTimestampingEnabled(view->timestampingCheckbox->getState());
}
