// tuneapp.


#include "InheritCheckItemView.h"

#include "../awt/Toolkit.h"

#include "../utils.h"

#include "../resource.h"

#include <assert.h>


InheritCheckItemView::InheritCheckItemView(CheckItem* item) :
		item(item), inheritor(dynamic_cast<Inheritable*>(item)) {
	assert (inheritor != NULL);
	createComponents();
}

void InheritCheckItemView::createComponents() {
	bool inherited = inheritor->isInherited();
	inheritCheckbox = new awt::Checkbox(getToolkit()->getStringResource(IDS_UNSET), inherited);
	checkbox = new awt::Checkbox(item->getLabel(), item->getCurrentValue());
	checkbox->setEnabled(!inherited);

	setLayout(utils::leftFlowLayout);
	add(checkbox);
	add(inheritCheckbox);

	inheritCheckbox->addItemListener(new InheritCheckboxListener(this));
	checkbox->addItemListener(new CheckboxListener(this));
	item->addPropertyChangeListener(new ItemPropertyChangeListener(this));
}

void InheritCheckItemView::InheritCheckboxListener::itemStateChanged(awt::ItemEvent& event) {
	bool inherit = target->inheritCheckbox->getState();
	target->inheritor->setInherited(inherit);
	target->checkbox->setEnabled(!inherit);
}

void InheritCheckItemView::CheckboxListener::itemStateChanged(awt::ItemEvent& event) {
	target->item->setCurrentValue(target->checkbox->getState());
}

void InheritCheckItemView::ItemPropertyChangeListener::propertyChange(PropertyChangeEvent& event) {
	const wstring& name = event.getPropertyName();
	if (name == Item::PROP_CURRENT_VALUE) {
		target->checkbox->setState(target->item->getCurrentValue());
	}
}

CheckItem* InheritCheckItemView::getTargetItem() {
	return item;
}
