// tuneapp.


#include "RecursiveListener.h"

#include <assert.h>


RecursiveListener::RecursiveListener() : target(NULL) {
}

void RecursiveListener::installTo(awt::Component* target) {
	assert (this->target == NULL);
	this->target = target;
	installRecursive(target);
}

void RecursiveListener::uninstall() {
	assert (target != NULL);
	uninstallRecursive(target);
	target = NULL;
}

void RecursiveListener::installRecursive(awt::Component* target) {
	installDispatcherTo(target);

	if (awt::Container* container = dynamic_cast<awt::Container*>(target)) {
		container->addContainerListener(this);
		for (	awt::Container::const_iterator children = container->begin();
				children != container->end();
				++children) {
			installRecursive(*children);
		}
	}
}

void RecursiveListener::uninstallRecursive(awt::Component* target) {
	uninstallDispatcherFrom(target);

	if (awt::Container* container = dynamic_cast<awt::Container*>(target)) {
		container->removeContainerListener(this);
		int count = container->getComponentCount();
		for (	awt::Container::const_iterator children = container->begin();
				children != container->end();
				++children) {
			uninstallRecursive(*children);
		}
	}
}

void RecursiveListener::componentAdded(awt::ContainerEvent& event) {
	awt::Component* child = event.getChild();
	installRecursive(child);
}

void RecursiveListener::componentRemoved(awt::ContainerEvent& event) {
	awt::Component* child = event.getChild();
	uninstallRecursive(child);
}
