// tuneapp.


#include "ValueItemView.h"

#include "../awt/Label.h"
#include "../awt/Toolkit.h"

#include "../utils.h"

#include "../resource.h"


ValueItemView::ValueItemView(ValueItem* item)
		: item(item), eventLockCount(0) {
	inheritable = dynamic_cast<Inheritable*>(item);
	if (inheritable && !inheritable->isInheritor()) {
		inheritable = NULL;
	}
	createComponents();
}

static inline int max3(int a, int b, int c) {
	return max(max(a, b), c);
}

void ValueItemView::createComponents() {
	numberField = new awt::NumberField();
	checkbox = new awt::Checkbox();

	int scale = item->getScale();
	if (item->getCurrentValue() == item->getDefaultValue()) {
		numberField->setValue(item->getDefaultMean() / scale);
	} else {
		numberField->setValue(item->getCurrentValue() / scale);
	}
	numberField->setMinimum(item->getMinimumValue() / scale);
	numberField->setMaximum(item->getMaximumValue() / scale);
	numberField->setUnitIncrement(item->getValueSteps() / scale);
	numberField->setBlockIncrement(item->getBlockSteps() / scale);
	numberField->setColumns(max3(
			5,
			toString(item->getMinimumValue() / scale).length() + 1,
			toString(item->getMaximumValue() / scale).length() + 1 ));

	if (inheritable) {
		checkbox->setLabel(getToolkit()->getStringResource(IDS_UNSET));
		checkbox->setState(inheritable->isInherited());
		numberField->setEnabled(!inheritable->isInherited());
	} else {
		checkbox->setLabel(getToolkit()->getStringResource(IDS_DEFAULT));
		bool isDefault = (item->getCurrentValue() == item->getDefaultValue());
		checkbox->setState(isDefault);
		checkbox->setEnabled(!isDefault);
	}

	setLayout(utils::leftFlowLayout);
	add(numberField);
	if (item->getUnit() != NULL_STRING) {
		add(new awt::Label(item->getUnit()));
	}
	add(checkbox);

	if (inheritable) {
		checkbox->addItemListener(new InheritCheckListener(this));
		item->addPropertyChangeListener(new ItemPropertyChangeListener(this));
	} else {
		checkbox->addItemListener(new DefaultCheckListener(this));
	}
	numberField->addAdjustmentListener(new NumberFieldListener(this));
}

void ValueItemView::InheritCheckListener::itemStateChanged(awt::ItemEvent& event) {
	Inheritable* inheritor = dynamic_cast<Inheritable*>(target->item);
	bool inherit = target->checkbox->getState();
	inheritor->setInherited(inherit);
	target->numberField->setEnabled(!inherit);
	if (inherit) {
		int value = target->item->getDefaultMean() / target->item->getScale();
		target->numberField->setValue(value);
	}
}

void ValueItemView::ItemPropertyChangeListener::propertyChange(PropertyChangeEvent& event) {
	const wstring& name = event.getPropertyName();
	if (name == Item::PROP_CURRENT_VALUE) {
		int value = target->item->getCurrentValue() / target->item->getScale();
		target->numberField->setValue(value);
	}
}

void ValueItemView::DefaultCheckListener::itemStateChanged(awt::ItemEvent& event) {
	if (target->eventLocked()) return;
	target->lockEvent();
	try {
		if (target->checkbox->getState()) {
			int value = target->item->getDefaultMean() / target->item->getScale();
			target->numberField->setValue(value);
			target->item->setDefault();
			target->checkbox->setEnabled(false);
			target->numberField->requestFocus();
		}
	} catch(...) {
		target->unlockEvent();
		throw;
	}
	target->unlockEvent();
}

void ValueItemView::NumberFieldListener::adjustmentValueChanged(awt::AdjustmentEvent& event) {
	if (target->eventLocked()) return;
	int value = event.getValue() * target->item->getScale();
	target->item->setCurrentValue(value);

	if (!target->inheritable) {
		target->lockEvent();
		try {
			bool isDefault = (
					(value == target->item->getDefaultValue()) ||
					(value == target->item->getDefaultMean()) );
			target->checkbox->setState(isDefault);
			target->checkbox->setEnabled(!isDefault);
		} catch(...) {
			target->unlockEvent();
			throw;
		}
		target->unlockEvent();
	}
}

ValueItem* ValueItemView::getTargetItem() {
	return item;
}

bool ValueItemView::eventLocked() {
	return (eventLockCount > 0);
}

void ValueItemView::lockEvent() {
	eventLockCount++;
}

void ValueItemView::unlockEvent() {
	eventLockCount--;
}
