// tuneapp.


#ifndef __VALUE_ITEM_VIEW_H__
#define __VALUE_ITEM_VIEW_H__


#include "../ValueItem.h"
#include "../Inheritable.h"

#include "../awt/Panel.h"
#include "../awt/Checkbox.h"
#include "../awt/NumberField.h"
#include "../awt/event/ItemListener.h"
#include "../awt/event/AdjustmentListener.h"
#include "../util/PropertyChangeListener.h"


class ValueItemView : public awt::Panel {
private:
	ValueItem* const item;
	Inheritable* inheritable;
	int eventLockCount;

	awt::Checkbox* checkbox;
	awt::NumberField* numberField;

public:
	ValueItemView(ValueItem* item);
	ValueItem* getTargetItem();

private:
	void createComponents();

	class InheritCheckListener : public awt::ItemListener {
	private:
		ValueItemView* const target;
	public:
		InheritCheckListener(ValueItemView* target) : target(target) {}
		void itemStateChanged(awt::ItemEvent& event);
	};
	friend class InheritCheckListener;

	class ItemPropertyChangeListener : public PropertyChangeListener {
	private:
		ValueItemView* const target;
	public:
		ItemPropertyChangeListener(ValueItemView* target) : target(target) {}
		void propertyChange(PropertyChangeEvent& event);
	};
	friend class ItemPropertyChangeListener;

	class DefaultCheckListener : public awt::ItemListener {
	private:
		ValueItemView* const target;
	public:
		DefaultCheckListener(ValueItemView* target) : target(target) {}
		void itemStateChanged(awt::ItemEvent& event);
	};
	friend class DefaultCheckListener;

	class NumberFieldListener : public awt::AdjustmentListener {
	private:
		ValueItemView* const target;
	public:
		NumberFieldListener(ValueItemView* target) : target(target) {}
		void adjustmentValueChanged(awt::AdjustmentEvent& event);
	};
	friend class NumberFieldListener;

	bool eventLocked();
	void lockEvent();
	void unlockEvent();
};


#endif // __VALUE_ITEM_VIEW_H__
