// tuneapp.


#ifndef __EVENT_LISTENER_LIST_H__
#define __EVENT_LISTENER_LIST_H__


#include <vector>
#include <algorithm>
using namespace std;


template<typename T>
class EventListenerList {
private:
	vector<T*> list;
	typedef vector<T*>::iterator iterator;

public:
	void add(T* listener) {
		if (find(list.begin(), list.end(), listener) == list.end()) {
			list.push_back(listener);
		}
	}

	void remove(T* listener) {
		vector<T*>::iterator found = find(list.begin(), list.end(), listener);
		if (found != list.end()) {
			list.erase(found);
		}
	}

	iterator begin() {
		return list.begin();
	}

	iterator end() {
		return list.end();
	}
};


template<typename T, typename E>
class listener_fun_t {
private:
	void (T::*pm)(E&);
	E& event;
public:
	listener_fun_t(void (T::*pm)(E&), E& event) : pm(pm), event(event) {}
	void operator()(T* listener) const {
		(listener->*pm)(event);
	}
};

template<typename T, typename E>
listener_fun_t<T, E> listener_fun(void (T::*pm)(E&), E& event) {
	return listener_fun_t<T, E>(pm, event);
};


#define FIRE_EVENT(LIST, LISTENER_CLASS, LISTENER_METHOD, EVENT_CLASS, EVENT) \
	for_each(LIST.begin(), LIST.end(), \
			listener_fun<LISTENER_CLASS, EVENT_CLASS>( \
			&LISTENER_CLASS::LISTENER_METHOD, EVENT) )


#endif // __EVENT_LISTENER_LIST_H__
