// tuneapp.


#ifndef __UTILS_H__
#define __UTILS_H__


#include "awt/FlowLayout.h"
#include "awt/GridLayout.h"
#include "awt/GridFlowLayout.h"

#include "Item.h"

#include "IOException.h"

#include <string>
using namespace std;


namespace utils {


extern awt::FlowLayout* leftFlowLayout;
extern awt::GridLayout* rowsGridLayout;
extern awt::GridFlowLayout* c2GridFlowLayout;

void init();


wstring getIDPath(const Item* item);

int hexToInt(wchar_t hex) throw(IllegalArgumentException);
wstring byteToHexString(signed char b);

wstring concatPath(const wstring& parent, const wstring& child);
wstring getAbsolutePath(const wstring& file);
wstring getFileBaseName(const wstring& path);
wstring getFileBasePath(const wstring& path);
int readFileAll(const wstring& fileName, wstring* buffer) throw(IOException);

wstring getWindowsDirectory();
wstring getSystemDirectory();

void exec(const wstring& command, const wstring& args) throw(IOException);


template<typename T>
class mem_vfun_t {
private:
	void (T::*pm)();
public:
	explicit inline mem_vfun_t(void (T::*pm)()) : pm(pm) {}
	inline void operator()(T* p) const {
		(p->*pm)();
	}
};

template<typename T>
inline mem_vfun_t<T> mem_vfun(void (T::*pm)()) {
	return mem_vfun_t<T>(pm);
};


template<typename K, typename T>
inline T find_ptr(const map<K, T>& m, const K& k) {
	map<K, T>::const_iterator f = m.find(k);
	return (f == m.end() ? NULL : (*f).second);
};


template<typename Func>
void for_items(const CategoryList& categoryList, Func& f) {
	for (	CategoryList::const_iterator iterator = categoryList.begin();
			iterator != categoryList.end();
			++iterator) {
		Category* category = *iterator;
		const ItemGroupList& groupList = category->getItemGroupList();
		for (	ItemGroupList::const_iterator groups = groupList.begin();
				groups != groupList.end();
				++groups) {
			ItemGroup* group = *groups;
			for (	ItemGroup::const_iterator items = group->begin();
					items != group->end();
					++items) {
				Item* item = *items;
				f(item);
			}
		}
		for_items(category->getSubCategoryList(), f);
	}
};


} // namespace


#endif // __UTILS_H__
