// Rick DOM.


#include "DocumentBuilder.h"
#include "Text.h"

#include <assert.h>

#ifdef _DEBUG
#include <windows.h>
#define debug(M) ::OutputDebugString(((M) + L"\r\n").c_str())
#else // _DEBUG
#define debug(M)
#endif


namespace xml {


DocumentBuilder::DocumentBuilder(Document* document)
		: document(document), currentNode(document) {
}

void DocumentBuilder::startElement(const wstring& name, const Attributes& attrs) throw(XMLException) {
	debug(L"startElement:" + name);
	Element* element = new Element(name);
	element->setAttributes(attrs);
	currentNode->add(element);
	currentNode = element;
}

void DocumentBuilder::endElement(const wstring& name) throw(XMLException) {
	debug(L"endElement:/" + name);
	if (currentNode->getName() != name) {
		throw XMLException(L"end name missmatch: " + currentNode->getName() + L" and /" + name);
	}
	currentNode = currentNode->getParent();
}

void DocumentBuilder::text(const wstring& text) throw(XMLException) {
	debug(L"text:" + text);
	Text* textNode = new Text(text);
	currentNode->add(textNode);
}

void DocumentBuilder::endDocument() throw(XMLException) {
	if (currentNode != document) {
		throw XMLException(L"no document end");
	}
}


} // namespace
