// Rick DOM.


#include "Element.h"

#include <assert.h>


namespace xml {


Element::Element(const wstring& name) : Node(ELEMENT, name) {
}

Element::Element(Type type, const wstring& name) : Node(type, name) {
}

Element::~Element() {
	for (	const_iterator iterator = begin();
			iterator != end();
			++iterator) {
		delete (*iterator);
	}
}

void Element::add(Node* node) throw(XMLException) {
	assert (node->parent == NULL);
	children.push_back(node);
	node->parent = this;
	textCache.resize(0);
}

int Element::getChildCount() const {
	return children.size();
}

Node::const_iterator Element::begin() const {
	return children.begin();
}

Node::const_iterator Element::end() const {
	return children.end();
}


void Element::setAttribute(const wstring& attr, const wstring& value) throw(XMLException) {
	attributes[attr] = value;
}

const wstring& Element::getAttribute(const wstring& attr) const {
	Attributes::const_iterator found = attributes.find(attr);
	if (found == attributes.end()) {
		return NULL_STRING;
	} else {
		return (*found).second;
	}
}

const Node::Attributes& Element::getAttributes() const {
	return attributes;
}

bool Element::hasAttribute(const wstring& attr) const {
	return (attributes.find(attr) != attributes.end());
}


const wstring& Element::getText() const {
	if (textCache.empty()) {
		for (	const_iterator iterator = begin();
				iterator != end();
				++iterator) {
			textCache += (*iterator)->getText();
		}
	}
	return textCache;
}


} // namespace
