// Rick DOM.


#include "Node.h"


#ifdef _DEBUG

#include <windows.h>
#include <sstream>

#endif // _DEBUG


namespace xml {


Node::Node(Type type, const wstring& name) : type(type), name(name), parent(NULL) {
}

Node::~Node() {
}

Node::Type Node::getType() const {
	return type;
}

const wstring& Node::getName() const {
	return name;
}

Node* Node::getParent() const {
	return parent;
}

bool Node::hasChild() const {
	return (getChildCount() > 0);
}

void Node::setAttributes(const Attributes& attrs) throw(XMLException) {
	for (	Attributes::const_iterator iterator = attrs.begin();
			iterator != attrs.end();
			++iterator) {
		setAttribute((*iterator).first, (*iterator).second);
	}
}


#ifdef _DEBUG

void Node::dump(int indent) {
	::OutputDebugString(wstring(indent, ' ').c_str());
	const type_info& info = typeid(*this);
	::OutputDebugStringA(info.name());
	wostringstream buffer;
	buffer	<< L"[name=" << name;
	const Attributes& attrs = getAttributes();
	for (	Attributes::const_iterator iterator = attrs.begin();
			iterator != attrs.end();
			++iterator) {
		buffer << L", " << (*iterator).first << L"=" << (*iterator).second;
	}
	buffer << L"]";
	::OutputDebugString(buffer.str().c_str());
	if (getChildCount() > 0) {
		::OutputDebugString(L" {\r\n");
		for (	const_iterator iterator = begin();
				iterator != end();
				++iterator) {
			(*iterator)->dump(indent + 2);
		}
		::OutputDebugString(wstring(indent, ' ').c_str());
		::OutputDebugString(L"}\r\n");
	} else {
		::OutputDebugString(L"\r\n");
	}
}

#endif // _DEBUG


} // namespace
