// Rick DOM.


#ifndef __XML_NODE_H__
#define __XML_NODE_H__


#include "XMLException.h"

#include <string>
#include <vector>
#include <map>
using namespace std;


namespace xml {


class Node {
public:
	enum Type {
		INVALID,
		DOCUMENT,
		ELEMENT,
		TEXT,
		CDATA_SECTION
	};

	typedef vector<Node*> NodeList;
	typedef NodeList::iterator iterator;
	typedef NodeList::const_iterator const_iterator;
	typedef map<wstring, wstring> Attributes;

protected:
	const Type type;
	const wstring name;
	Node* parent;

private:
	Node(const Node&); // no copy
	Node& operator=(const Node&); // no copy
protected:
	Node(Type type, const wstring& name);
public:
	virtual ~Node();

	const wstring& getName() const;
	Type getType() const;
	Node* getParent() const;

	virtual void add(Node* node) throw(Exception) = 0;
	virtual int getChildCount() const = 0;
	bool hasChild() const;
	virtual const_iterator begin() const = 0;
	virtual const_iterator end() const = 0;

	virtual void setAttribute(const wstring& attr, const wstring& value) throw(Exception) = 0;
	void setAttributes(const Attributes& attrs) throw(Exception);
	virtual const wstring& getAttribute(const wstring& attr) const = 0;
	virtual const Attributes& getAttributes() const = 0;
	virtual bool hasAttribute(const wstring& attr) const = 0;

	virtual const wstring& getText() const = 0;

	friend class Element;

#ifdef _DEBUG
public:
	void dump(int indent = 0);
#endif // _DEBUG
};


} // namespace


#endif // __XML_NODE_H__
