// Rick DOM.


#ifndef __XML_PARSER_H__
#define __XML_PARSER_H__


#include "ParserCallback.h"
#include "EntityTable.h"

#include "../IOException.h"

#include <istream>
using namespace std;


namespace xml {


class Parser  {
protected:
	ParserCallback* const callback;

	EntityTable entityTable;
	typedef std::map<std::wstring, std::wstring> Attributes;

public:
	Parser(ParserCallback* callback);
	void parse(std::wistream& in) throw(XMLException);

protected:
	enum Type {
		START_TAG,
		END_TAG,
		EMPTY_TAG,
		COMMENT,
		PROCESSING_INSTRUCTION,
		CDATA_SECTION,
		SECTION
	};

protected:
	void readXmlDeclaration(std::wistream& in);
	void readElements(std::wistream& in);
	std::wistream::int_type skipWhiteSpace(std::wistream& in);
	Type readTagLikeStructure(std::wistream& in, std::wstring& name, Attributes& attrs, std::wstring& extra);
	void readTagLikeStructureAfterLt(std::wistream& in);
	Type readTagLikeStructureAfterLt(std::wistream& in, std::wstring& name, Attributes& attrs, std::wstring& extra);
	void readProcessingInstructionAfterQ(std::wistream& in, std::wstring& name, Attributes& attrs);
	Type readSectionLikeStructureAfterEx(std::wistream& in, std::wstring& name, Attributes& attrs, std::wstring& extra);
	void readTagAfter(std::wistream& in, wchar_t preC, std::wstring& name, Attributes& attrs, const std::wstring& endMark, bool& empty);
	void readAttributes(std::wistream& in, Attributes& attrs, const std::wstring& endMark, bool& empty);
	bool readAttribute(std::wistream& in, std::wstring& attrName, std::wstring& value, const std::wstring& endMark, bool& empty);
	bool readAttributeValue(std::wistream& in, std::wstring& value, const std::wstring& endMark, bool& empty);
	void perhapsTagEnd(std::wistream& in, wchar_t preC, const std::wstring& endMark);
	void readCommentAfterHyph(std::wistream& in, wchar_t preC, std::wstring& name, std::wstring& extra);
	Type readConditionalSectionAfterBracket(std::wistream& in, std::wstring& name, std::wstring& extra);
	void readCdataSection(std::wistream& in, std::wstring& extra);
	void readSectionAfter(std::wistream& in, wchar_t preC, std::wstring& name, std::wstring& extra);
	void readUntil(std::wistream& in, std::wstring& extra, const std::wstring& endMark);
	void readTextAfter(std::wistream& in, wchar_t preC);

	void readReferenceAfterAmp(std::wistream& in, std::wstring& dest);
	wchar_t readNumericReferenceAfterSharp(std::wistream in);
	void readEntityReferenceAfter(std::wistream& in, wchar_t preC, std::wstring& dest);
};


} // namespace


#endif // __XML_PARSER_H__
