// tuneapp2 build tools.


import org.apache.tools.ant.taskdefs.MatchingTask;
import org.apache.tools.ant.types.FileSet;
import org.apache.tools.ant.DirectoryScanner;
import org.apache.tools.ant.util.SourceFileScanner;
import org.apache.tools.ant.util.IdentityMapper;

import org.apache.tools.ant.BuildException;

import java.io.*;

import java.util.List;
import java.util.ArrayList;
import java.util.Iterator;

/**
 *	Charset converter (ant task).
 */
public class CharConv extends MatchingTask {
	private List filesets = new ArrayList();
	private File destDir = null;
	private String inputEncoding = null;
	private String outputEncoding = null;
    private boolean quiet = true;

	public void addFileset(FileSet fileset) {
		this.filesets.add(fileset);
	}

	public void setDestdir(File destDir) {
		this.destDir = destDir;
	}

	public void setInputEncoding(String inputEncoding) {
		this.inputEncoding = inputEncoding;
	}

	public void setOutputEncoding(String outputEncoding) {
		this.outputEncoding = outputEncoding;
	}

	public void setQuiet(boolean quiet) {
		this.quiet = quiet;
	}

	public void execute() throws BuildException {
		if (destDir == null) {
			throw new BuildException("The destdir attribute must be set.");
		}
		destDir.mkdirs();

		List convertingList = new ArrayList(); // List<File[] { src, dest }>
		Iterator iterator = filesets.iterator();
		while (iterator.hasNext()) {
			FileSet fileset = (FileSet)iterator.next();
			File srcDir = fileset.getDir(getProject());
	        DirectoryScanner scanner = fileset.getDirectoryScanner(getProject());
			String[] srcFiles = scanner.getIncludedFiles();
			SourceFileScanner sfs = new SourceFileScanner(this);
			srcFiles = sfs.restrict(srcFiles, srcDir, destDir, new IdentityMapper());
			for (int i = 0; i < srcFiles.length; i++) {
				File[] entry = new File[] {
					new File(srcDir, srcFiles[i]),
					new File(destDir, srcFiles[i]),
				};
				convertingList.add(entry);
			}
		}

		int length = convertingList.size();
		if (length == 0) return;
        String message
				= "Converting " + length + " file" + (length != 1 ? "s" : "")
				+ " to " + destDir;
        log(message);
		iterator = convertingList.iterator();
        while (iterator.hasNext()) {
			File[] entry = (File[])iterator.next();
			try {
	            convert(entry[0], entry[1]);
			} catch (IOException e) {
				throw new BuildException(e);
			}
        }
	}

	protected void convert(File srcFile, File destFile) throws IOException {
		InputStream is = null;
		destFile.getParentFile().mkdirs();
		OutputStream os = new BufferedOutputStream(new FileOutputStream(destFile));
		try {
			is = new BufferedInputStream(new FileInputStream(srcFile));
			Reader in;
			if (inputEncoding == null) {
				in = new InputStreamReader(is);
			} else {
				in = new InputStreamReader(is, inputEncoding);
			}
			Writer out;
			if (outputEncoding == null) {
				out = new OutputStreamWriter(os);
			} else {
				out = new OutputStreamWriter(os, outputEncoding);
			}
			char[] buffer = new char[8 * 1024];
			int length;
			while ((length = in.read(buffer)) != -1) {
				out.write(buffer, 0, length);
			}
			out.close();
			in.close();
		} finally {
			if (is != null) is.close();
			os.close();
		}
	}
}
